/*--------------------------------------------------------------------------*\
	FILE: SAMP15.C  																					

	DESCIPTION:
		 USING SPECIAL EFFECTS FEATURES

	   The calls demonstrated here are:

		program and the steps taken to add PG SDK calls to applications in general.

	WRITTEN BY: Dan Weingart

	DATE: October 28, 1992

	PG VERSION: 1.00.3

	OTHER FILES NEEDED TO COMPILE:
		SAMP15.MAK
		SAMP15.H
		SAMP15.DEF
		PG.H
		PG.LIB

		SAMP15.RC		* RESOURCE FILE *
		TDG_LOGO.ICO	* RESOURCE FILE *

	OTHER FILES NEEDED TO RUN:
		DEFAULT.TDL
\*--------------------------------------------------------------------------*/

#include <Windows.H>
#include <stdlib.h>
#include "SAMP15.H"

#include "pg.h"		/* PG SDK GRAPHICS LIBRARY */
#include "pg_fx.h"	/* PG SPECIAL EFFECTS EXTENSIONS */

/*--------------------------------------------------------------------------*\
|            	         FUNCTION PROTOTYPES                           		 |
\*--------------------------------------------------------------------------*/

long	FAR	PASCAL	RecWndProc (HWND, UINT, UINT, LONG);

VOID NEAR PASCAL XOR_Item (HWND hWnd, SelItemPtr pItem);
VOID NEAR PASCAL XOR_List (HWND hWnd, SelListPtr pSelList);

VOID	NEAR	PASCAL	UpdateMenu  (HMENU hmenu, WORD *mVar, WORD wParam);

/* PROGRAM SPECIFIC CALLS TO HOOK UP PG SDK LIBRARY TO LOCAL DATA SOURCES */
BOOL PUBLIC InitDataCallBacks(HANDLE hInst, GraphPtr pGraph);
VOID PUBLIC FiniDataCallBacks(GraphPtr pGraph);
INT16 FAR PASCAL LOCAL_GetDataCallback(INT16	nDataPt,INT16 nSeries,
												  	REAL64 FAR * prVal,INT32 lClientData);

/*--------------------------------------------------------------------------*\
|            	         Static Data.                                  		 |
\*--------------------------------------------------------------------------*/

HANDLE  hInst;

/* PG - Global graph variables */
DrawEnvPtr	   gpDrawEnv;						/* ptr to StdDraw Environment */
GraphPtr		   gpGraph;							/* ptr to Graph Object */
int				gbGraphValid;					/* TRUE when graph exists */

SelListPtr		gpList;							/* ptr to Selection List */
DetNodeRef		gDetNode;						/* global DetNode			 */
DetNodeRefPtr	gpDetNode = &gDetNode;		/* ptr to global DetNode */
SelItemPtr		gpSelItem;						/* ptr to Selection Item */

/* THE FOLLOWING STATIC VARIABLES ARE USED TO STORE THE CHART DATA */
#define MaxStringSz	64

char	StringData[11][MaxStringSz] = {"PG Sample Chart",
												 "by Palisades Software",
												 "Dan", "Elmer","Jon",
												 "May", "June", "July",
												 "The Row Title", "The Column Title",
												 "A footnote"};

int   NumData [3][3] = {{300,420,500},
								{400,500,600},
								{500,640,700}};




/*--------------------------------------------------------------------------*\
|            	         Main Function: WinMain                        		 |
\*--------------------------------------------------------------------------*/

int PASCAL WinMain	(HANDLE hInstance,
							 HANDLE hPrevInstance,
							 LPSTR  lpszCmdLine,
							 int	  cmdShow)

	{
	HWND		hwnd;
	MSG		msg;
	WNDCLASS	wndclass;
	INT16	  	nStatus;
	char	  	tmpbuf[256];

	if (!hPrevInstance)
	  {
	  wndclass.lpszClassName = "SAMP15:MAIN";
	  wndclass.hInstance     = hInstance;
 	  wndclass.lpfnWndProc	 = RecWndProc;
	  wndclass.hCursor       = LoadCursor(NULL, IDC_ARROW);
	  wndclass.hIcon		    = LoadIcon (hInstance, "TDG_LOGO");
	  wndclass.lpszMenuName  = "SAMP15_MENU";
	  wndclass.hbrBackground = COLOR_WINDOW+1;
	  wndclass.style		    = CS_HREDRAW | CS_VREDRAW;
	  wndclass.cbClsExtra	 = 0;
	  wndclass.cbWndExtra	 = 0;

	  RegisterClass( &wndclass);
	  }

	hInst = hInstance;


	if ( nStatus = InitTDGLIB() ) 		/* PG - MAIN INITIALIZATION CALL */
	   {
		wsprintf(tmpbuf, "InitTDGLIB failed (%d)!", nStatus);
		MessageBox(NULL, tmpbuf, "WinMain",
					  MB_OK | MB_ICONHAND | MB_SYSTEMMODAL );
	   }

	hwnd = CreateWindow("SAMP15:MAIN", 			/* Class Name. */
							  "PG Sample #15 F/X",		/* Title			*/
							  WS_OVERLAPPEDWINDOW,	/* Style bits. */
							  CW_USEDEFAULT,			/* x - default */
							  0,							/* y - default	*/
							  CW_USEDEFAULT,			/* cx - default*/
							  0,							/* cy - default*/
							  NULL,						/* No parent	*/
							  NULL,						/* Class menu. */
							  hInstance,				/* Creator		*/
							  NULL						/* Params		*/
							 ) ;

	ShowWindow (hwnd, cmdShow);


  if ( !(gpGraph = AllocGraphPtr()) ) /* PG - ALLOCATE A GRAPH POINTER */
		return FALSE;
						
	/* THE NEXT CALL IS A LOCAL PROC THAT CALLS THE PG ROUTINES FOR
	   HOOKING UP A LOCAL DATA SOURCE TO THE PG LIBRARY */

	InitDataCallBacks(hInst, gpGraph);  

													/* PG - ALLOCATE DRAW ENVIRONMENT */
  if ( !(gpDrawEnv = AllocDrawEnvPtr(hwnd, DE_PORT_NORMAL)) )	
		return FALSE;
													/* PG- LOAD A BAR GRAPH TEMPLATE AS DEFAULT */
  nStatus = TDIReadGraph(gpGraph, REQUEST_TDL, "default.TDL");
	if (nStatus != E00_OK)
		 {
		 MessageBeep(-1);
		 MessageBeep(-1);
		 MessageBeep(-1);
		 //dprintf("TDIReadGraph returned error code: %d\n",nStatus);
//	   return FALSE;
		 }
													/* PG- CREATE AN EMPTY SELECTION LIST */
	gpList = Select_AllocList();

  UpdateWindow(hwnd);             /* Sends WM_PAINT message */

	while (GetMessage (&msg, 0, 0, 0))
		{
		TranslateMessage (&msg);	  /* Keyboard input. */
		DispatchMessage (&msg);
		}


	/* PERFORM PG CLEAN-UP */

	Select_FreeList (gpList);		 /* PG - FREE MEMORY USED BY THIS POINTER */

	FreeDrawEnvPtr(gpDrawEnv);	 /* PG - FREE MEMORY USED BY THIS POINTER */

	FiniDataCallBacks(gpGraph);	 /* LOCAL PROC - CLEAN UP THUNKS */
	
	FreeGraphPtr(gpGraph);			 /* PG - FREE MEMORY USED BY THIS POINTER */

  FiniTDGLIB(); 							 /* PG - CLEAN-UP */

	return 0;
	}


/*--------------------------------------------------------------------------*\
|            	         Window Procedure: RecWndProc.                 		 |
|   This is the main message processing loop for this application           |
\*--------------------------------------------------------------------------*/

long FAR PASCAL RecWndProc (HWND hWnd,		UINT msg,
									 UINT wParam,	LONG lParam)

	{								

	static RECT	 r;

	static WORD	 wEffect = IDM_NORMAL;
	static WORD	 wPicto	 = IDM_PICTO_OFF;

 	/* PG - The following is a table of frequently used colors */

#define FULL 65535		/* PG - FULL INTENSITY OF A GIVEN COLOR */

	static RGB16 aRGB[5]  = {{   0,   0,   0},	/* BLACK */
									 {FULL,FULL,FULL},	/* WHITE */
									 {FULL,   0,   0},	/* RED   */
									 {   0,FULL,   0},	/* GREEN */
									 {   0,   0,FULL}};	/* BLUE  */

	HMENU	hmenu;
	char 	buffer[80];
	static int 	i = 1;
	BOOL   bTRUE = TRUE;
	BOOL	 bFALSE= FALSE;


	switch (msg)
		{
		case WM_COMMAND:
			hmenu = GetMenu (hWnd);
			switch (wParam)
				{
				case IDM_QUIT:			   /* TIME TO QUIT! */
					DestroyWindow (hWnd);
					break;

			  case IDM_BLACK:		
			  case IDM_WHITE:		
			  case IDM_RED:							
			  case IDM_GREEN:		
			  case IDM_BLUE:
					i = wParam - IDM_BLACK;
					SetGraphAttrSL(gpGraph,gpList,A_AREACOLOR_RGB,&aRGB[i]);
					SetGraphAttrSL(gpGraph,gpList,A_FONTCOLOR_RGB,&aRGB[i]);
					InvalidateRect (hWnd, &r, TRUE);
					UpdateWindow(hWnd);
					break;

				case IDM_NORMAL:
					{
					RGB16	aRGB;

					UpdateMenu (hmenu,&wEffect,wParam);		
					GetGraphAttrSL(gpGraph,gpList,A_AREACOLOR_RGB,&aRGB);

					/* SETTING A COLOR OR PATTERN WILL REMOVE F/X */
					SetGraphAttrSL(gpGraph,gpList,A_AREACOLOR_RGB,&aRGB);
			  	SetGraphAttrSL (gpGraph,gpList, A_LINECOLOR_NONE, &bFALSE);
					InvalidateRect (hWnd, &r, TRUE);
					UpdateWindow(hWnd);  
					}
					break;


				case IDM_WASH:
					{
					FXStruct	aSFX;

					aSFX.effect = WASH_FX;
					aSFX.info.wash.startColor = aRGB[4];	/* BLUE 	*/
					aSFX.info.wash.endColor 	= aRGB[0];	/* BLACK 	*/
					aSFX.info.wash.washType 	= 1;				/* AB     */
					aSFX.info.wash.washDir  	= WashDown;
					aSFX.info.wash.washScale 	= 0;				/* MAC ONLY */
					aSFX.info.wash.startBin 	= 0;				/* MAC ONLY */
					aSFX.info.wash.endBin 		= 0;				/* MAC ONLY */

					UpdateMenu (hmenu,&wEffect,wParam);		
					SetGraphAttrSL(gpGraph,gpList,A_AREASFX,&aSFX);
					InvalidateRect (hWnd, &r, TRUE);
					UpdateWindow(hWnd);  
					}
 					break;


				case IDM_WASH2:
					{
					FXStruct	aSFX;

					aSFX.effect = WASH_FX;
					aSFX.info.wash.startColor = aRGB[i];	/* GREEN 	*/
					aSFX.info.wash.endColor 	= aRGB[0];	/* BLACK 	*/
					aSFX.info.wash.washType 	= 3;				/* ABA    */
					aSFX.info.wash.washDir  	= WashLeft;
					aSFX.info.wash.washScale 	= 0;				/* MAC ONLY */
					aSFX.info.wash.startBin 	= 0;				/* MAC ONLY */
					aSFX.info.wash.endBin 		= 0;				/* MAC ONLY */

					UpdateMenu (hmenu,&wEffect,wParam);		
					SetGraphAttrSL(gpGraph,gpList,A_AREASFX,&aSFX);
					InvalidateRect (hWnd, &r, TRUE);
					UpdateWindow(hWnd);  
					}
 					break;


				case IDM_PICTURE:
					{
					FXStruct	aSFX;

					aSFX.effect = PICTURE_FX;
					aSFX.info.picture.pictScale = Scale2Fit;
					aSFX.info.picture.pictFlip  = NullFlip;
					aSFX.info.picture.xExt		  = 0;									/* METAFILE ONLY */
					aSFX.info.picture.yExt		  = 0;									/* METAFILE ONLY */
					aSFX.info.picture.format	  = 0;
					aSFX.info.picture.hdl			  = NULL;


					/* To load a bitmap from a resource, format the string as follows:
							"#<NAME OF RESOURCE>#<NAME OF CALLING MODULE>" */
					lstrcpy(aSFX.info.picture.pictName,"#TESTBMP#SAMP15.EXE"); 


					UpdateMenu (hmenu,&wEffect,wParam);		
					SetGraphAttrSL(gpGraph,gpList,A_AREASFX,&aSFX);
					InvalidateRect (hWnd, &r, TRUE);
					UpdateWindow(hWnd);  
					}
					break;

				case IDM_PICTURE2:
					{
					FXStruct	aSFX;

					aSFX.effect = PICTURE_FX;
					aSFX.info.picture.pictScale = Tiled;
					aSFX.info.picture.pictFlip  = NullFlip;
					aSFX.info.picture.xExt		  = 0;									/* METAFILE ONLY */
					aSFX.info.picture.yExt		  = 0;									/* METAFILE ONLY */
					aSFX.info.picture.format	  = 0;
					aSFX.info.picture.hdl			  = NULL;
					lstrcpy(aSFX.info.picture.pictName,"SAMP15.BMP");

					UpdateMenu (hmenu,&wEffect,wParam);		
					SetGraphAttrSL(gpGraph,gpList,A_AREASFX,&aSFX);
					InvalidateRect (hWnd, &r, TRUE);
					UpdateWindow(hWnd);  
					}
					break;

				case IDM_VECTOR:
				  {
					FXStruct	aSFX;

					aSFX.effect = PICTURE_FX;
					aSFX.info.picture.pictScale = Scale2Fit;
					aSFX.info.picture.pictFlip  = NullFlip;
					aSFX.info.picture.xExt		  = 0;			 				/* METAFILE ONLY */
					aSFX.info.picture.yExt		  = 0;			 				/* METAFILE ONLY */
					aSFX.info.picture.format	  = 0;
					aSFX.info.picture.hdl			  = NULL;
					lstrcpy(aSFX.info.picture.pictName,"HAPPY.WMF");

					UpdateMenu (hmenu,&wEffect,wParam);		
					SetGraphAttrSL(gpGraph,gpList,A_AREASFX,&aSFX);
			  	SetGraphAttrSL (gpGraph,gpList, A_LINECOLOR_NONE, &bTRUE);
					InvalidateRect (hWnd, &r, TRUE);
					UpdateWindow(hWnd);  
					}
					break;

				case IDM_SHADOW:
					{
					DropShadowInstRec Shadow;

					Shadow.nXoff = 100;
					Shadow.nYoff = 120;
					Shadow.nRed 	= 0;
					Shadow.nGreen = 0;
					Shadow.nBlue 	= 0;
					UpdateMenu (hmenu,&wEffect,wParam);		
					SetGraphAttrSL(gpGraph,gpList,A_AREADROPSHADOW,&Shadow);
					SetGraphAttrSL(gpGraph,gpList,A_FONTDROPSHADOW,&Shadow);
					InvalidateRect (hWnd, &r, TRUE);
					UpdateWindow(hWnd);  
					}
					break;

				case IDM_SHADOW2:
					{
					DropShadowInstRec Shadow;

					Shadow.nXoff = 300;
					Shadow.nYoff = 350;
					Shadow.nRed 	= FULL/2;
					Shadow.nGreen = FULL/2;
					Shadow.nBlue 	= FULL/2;
					UpdateMenu (hmenu,&wEffect,wParam);		
					SetGraphAttrSL(gpGraph,gpList,A_AREADROPSHADOW,&Shadow);
					SetGraphAttrSL(gpGraph,gpList,A_FONTDROPSHADOW,&Shadow);
					InvalidateRect (hWnd, &r, TRUE);
					UpdateWindow(hWnd);  
					}
					break;

				case IDM_SHADOW_OFF:
					{
					DropShadowInstRec Shadow;

					Shadow.nXoff = 0;
					Shadow.nYoff = 0;
					Shadow.nRed 	= 0;
					Shadow.nGreen = 0;
					Shadow.nBlue 	= 0;
					UpdateMenu (hmenu,&wEffect,wParam);		
					SetGraphAttrSL(gpGraph,gpList,A_AREADROPSHADOW,&Shadow);
					SetGraphAttrSL(gpGraph,gpList,A_FONTDROPSHADOW,&Shadow);
					InvalidateRect (hWnd, &r, TRUE);
					UpdateWindow(hWnd);  
					}
					break;

				case IDM_PICTO_ON:
					UpdateMenu (hmenu,&wPicto,wParam);		
					SetGraphAttrSL(gpGraph,NULL,ABA_PICTOGRAPH,&bTRUE);
					InvalidateRect (hWnd, &r, TRUE);
					UpdateWindow(hWnd);  
					break;

				case IDM_PICTO_OFF:
					UpdateMenu (hmenu,&wPicto,wParam);		
			  	SetGraphAttrSL (gpGraph,gpList, A_LINECOLOR_NONE, &bFALSE);
					SetGraphAttrSL(gpGraph,NULL,ABA_PICTOGRAPH,&bFALSE);
					InvalidateRect (hWnd, &r, TRUE);
					UpdateWindow(hWnd);  
					break;

				default:
					wsprintf (buffer, "Command = %d", wParam);
					MessageBox (hWnd, buffer, "WM_COMMAND", MB_OK);

				}
			break;
		case WM_LBUTTONDOWN:
			{
			POINT pt;
			POINTS pts;

			pts = MAKEPOINTS (lParam);	/* Get the Mouse Pt. */
			pt.x = pts.x;
			pt.y = pts.y;

			dvPoint(gpDrawEnv,&pt);		/*PG - Convert to Virt. Coord's. */

												/*PG - See if this point corresponds	to
										           any Detectable region in our graph */
		   if (FindDetNode (gpGraph, &pt, gpDetNode)) 
			  {

			  /* IF SHIFT KEY */

			  if (MK_SHIFT & wParam)
			    {								/*PG - See if this item is already in
															our master list */
			    gpSelItem = Select_SearchList (gpList, gpDetNode);
			    if (gpSelItem)
					{
				   XOR_Item(hWnd,gpSelItem);	/*See sub-routine for PG XOR call */
											  /*PG - Remove this item from our list */
				   Select_RemoveItem (gpList, gpSelItem);
					}
			    else
					{						  /*PG - Add this item to our list */
				   gpSelItem = Select_AddItem (gpGraph,gpList,gpDetNode);
											  /*PG - Calculate Sizing Handles for item */
					Select_CalcHandlesItem(gpDrawEnv,gpSelItem);
					XOR_Item(hWnd,gpSelItem); /*See sub-routine for PG XOR call */
					}
			    }

			  /* NO SHIFT KEY */

			  else
			    {
				 XOR_List(hWnd,gpList);		  /*See sub-routine for PG XOR call */	
				 Select_ClearList	(gpList);  /*PG - Clear our master list */
											  /*PG - Add this item to our list */
				 gpSelItem = Select_AddItem(gpGraph,gpList,gpDetNode);
				 							  /*PG - Calculate Sizing Handles for item */
				 Select_CalcHandlesItem(gpDrawEnv,gpSelItem);
				 XOR_Item(hWnd,gpSelItem);
			    }
			  }
			}

			break;


		case WM_DESTROY:
			PostQuitMessage (0);
			break;

		case WM_PAINT:
			{
			PAINTSTRUCT ps;
			HDC hdc;
			int i;

          XOR_List(hWnd,gpList);			/*PG - TURN XOR HANDLES OFF */

			hdc = BeginPaint (hWnd, &ps);

   	/*PG - SET GRAPH REGION TO ENTIRE CLIENT AREA */
			    GetClientRect(hWnd, &r);
			    DESetDestRect(gpDrawEnv, &r);

   	/*PG - SET DRAWING MODE TO NORMAL, ON-SCREEN */
			    DESetPortInfo(gpDrawEnv, hdc, DE_PORT_NORMAL);

	   /*PG - DRAW THE GRAPH */
          DrawBegin(gpDrawEnv);
	          DrawTheGraph(gpDrawEnv, gpGraph, TRUE, TRUE, TRUE);
			    DrawEnd(gpDrawEnv);

	   /* PG - Match up objects in selection list with new detnodes */
					Select_RebuildList (gpGraph,gpList);

	   /* PG - Calculate handles for objects in selection list based on new detnodes*/
					Select_CalcHandlesList (gpDrawEnv,gpList);

		   EndPaint (hWnd, &ps);

       XOR_List(hWnd,gpList);			/*See subroutine for PG call */

		   }
			break;

		default:
			return(DefWindowProc(hWnd,msg,wParam,lParam));
			break;
		}
	return 0L;
	}


/*--------------------------------------------------------------------------*\
|            	  XOR_Item
\*--------------------------------------------------------------------------*/

void NEAR PASCAL XOR_Item (HWND hWnd, SelItemPtr pItem)
	{
	HDC hdc;

	hdc = GetDC (hWnd);
					/*PG- set to normal drawing mode */
  DESetPortInfo(gpDrawEnv, hdc, DE_PORT_NORMAL);
					/*PG- Begin PG Drawing */
  DrawBegin(gpDrawEnv);
					/*PG- XOR the boundaries of an item in a selection list */
	Select_DrawXorItem(gpDrawEnv, pItem);
					/*PG- End PG Drawing */
  DrawEnd(gpDrawEnv);
	ReleaseDC (hWnd,hdc);
	}

/*--------------------------------------------------------------------------*\
|            	  XOR_List
\*--------------------------------------------------------------------------*/

void NEAR PASCAL XOR_List (HWND hWnd, SelListPtr pSelList)
	{
	HDC hdc;

	hdc = GetDC (hWnd);
  DESetPortInfo(gpDrawEnv, hdc, DE_PORT_NORMAL);
  DrawBegin(gpDrawEnv);
					/*PG- XOR the boundaries of EVERY item in a selection list */
	Select_DrawXorList(gpDrawEnv, pSelList);
  DrawEnd(gpDrawEnv);
	ReleaseDC (hWnd,hdc);
	}


/*-------------------DATA-CALLBACK-ROUTINES---------------------------------*/

/*--------------------------------------------------------------------------*\
|            	  LOCAL_GetStringCallback.						              		 |
| Gets asked by PG for a particular piece of legend text. Returns the 	 |
| correct string by looking into the StringData array.                      |   
| The gs constants (gsTITLE,gsSUBTITLE) are enumerated in PG.H				 |
\*--------------------------------------------------------------------------*/

INT16 FAR PASCAL  
LOCAL_GetStringCallback(
	INT16						nWhich,							/* Which legend?. */
	char FAR *			   lpBuffer,						/* Put answer here */
	INT16						nDimBuffer,						/* Max.size of answer */
	INT32						lClientData						/* client data */
)
{
switch (nWhich)
  {
  case gsTITLE:
    lstrcpy (lpBuffer,StringData[0]);	/* RETURN THE TITLE OF THE GRAPH */
	 break;
  case gsSUBTITLE:							/* RETURN THE SUBTITLE OF THE GRAPH */
    lstrcpy (lpBuffer,StringData[1]);
	 break;
  case gsFOOTNOTE:							/* RETURN THE GRAPH FOOTONOTE */
    lstrcpy (lpBuffer,StringData[10]);
	 break;
  case gsSERIESTITLE:						/* RETURN THE SERIES TITLE */
    lstrcpy (lpBuffer,StringData[9]);
	 break;
  case gsGROUPSTITLE:						/* RETURN THE GROUP TITLE */
    lstrcpy (lpBuffer,StringData[8]);
	 break;
  default:										/* PUT "XXXX" IN ANY OTHER REQUEST */
    lstrcpy (lpBuffer,"XXXX");
  }
return TRUE;
}


/*--------------------------------------------------------------------------*\
|            	  LOCAL_GetLabelCallback.						              		 |
| Gets asked by PG either a row or column header string. Returns the 	    |
| correct string by looking into the StringData array.                      |   
\*--------------------------------------------------------------------------*/
INT16 FAR PASCAL 
LOCAL_GetLabelCallback(
	INT16						nCol,				/* Set to -1 when traversing Row headers */
	INT16						nRow,				/* Set to -1 when traversing Col headers */ 
	char FAR *		   	lpBuffer,		/* Put answer here. */
	INT16						nDimBuffer,		/* Max.size of answer. */
	INT32						lClientData		/* client data value */
)
{
int i;

if (nCol == -1) 	/* THIS MEANS WE'RE TRAVERSING ROW HEADERS */
  i = 2 + nRow;
else
  if (nRow == -1)	/* THIS MEANS WE'RE TRAVERSING COL HEADERS */
    i = 5 + nCol; /* OTHERWISE WE'RE TRAVERSING COLUMN HEADERS */
  else
    i = 10;			/* ANY OTHER REQUEST GETS THE FOOTNOTE STRING */

lstrcpy (lpBuffer,StringData[i]);	/* RETURN THE CORRECT HEADER */

return TRUE;
}


/*--------------------------------------------------------------------------*\
|            	  LOCAL_GetDataCallback.						              		    |
| Gets asked by PG for a piece of numeric data corresponding to a			 |
| particular row & column offset.  Returns the 	                            |
| correct value by looking into the NumData array.                          |   
\*--------------------------------------------------------------------------*/

INT16 FAR PASCAL 
LOCAL_GetDataCallback(
	INT16						nDataPt,							/* Row number of requested value. */
	INT16						nSeries,							/* Column number of requested value. */
	REAL64 FAR *		   prVal,							/* Put answer here   */
	INT32						lClientData						/* client data value */
)
{
*prVal = (REAL64) NumData [nDataPt][nSeries];      /* RETURN THE VALUE */
return TRUE;
}


/*--------------------------------------------------------------------------*\
|            	  LOCAL_GetDataCallback.						          		    |
| Utility function that formats a numeric value of type REAL64 into a		 |
| string. Not used by any of the graphs in this program.							 |
\*--------------------------------------------------------------------------*/

INT16 FAR PASCAL 
LOCAL_RealToStrCallback(
	REAL64					rVal,								/* Value to be formatted. */
	char FAR *			lpszResult,							/* Caller provided buffer for string assigment. */
	INT16						nDimBuffer,						/* Buffer length. */
	INT16						nFmtCode,						/* Format conversion identifier. */
	INT32						lClientData						/* client data value */
)
{
wsprintf(lpszResult,"%d",(int) rVal);	 		/* JUST DO SIMPLE INT FORMAT */
return TRUE;
}

/*--------------------------------------------------------------------------*\
|            	  LOCAL_SetPageCallback.						           		    |
| Not actually used by current PG, but needed for expansion.  For now,    |
| just return FALSE 																			 |
\*--------------------------------------------------------------------------*/

INT16 FAR PASCAL 
LOCAL_SetPageCallback(
	INT16						nPage,								/* New current page. */
	INT32						lClientData						/* client data value */
)
{
return FALSE;
}

/************************************************************************
* 	InitDataCallBacks
*
*	DESCRIPTION:
*		This function connects the above CallBack functions with a 
*		particular graph.
*		
*	RETURNS:
*		FALSE if any of the callback functions are set to NULL.
*
************************************************************************/

BOOL PUBLIC InitDataCallBacks(HANDLE hInst, GraphPtr pGraph)
{
	GraphDataInfo		graphData;
	
	/*
	** PG - Look up Data information connected to this graph
   */
	GetGraphDataInfo(pGraph, &graphData);
	
	/*
	** If the data information doesn't exist, create thunks as necessary
	*/
	if (!graphData.lpfnGetData)
	   { 
		graphData.lpfnGetString    = (LPFNGetStringCallBack)    MakeProcInstance((FARPROC) LOCAL_GetStringCallback, hInst);
		graphData.lpfnGetLabel     = (LPFNGetLabelCallBack)     MakeProcInstance((FARPROC) LOCAL_GetLabelCallback,  hInst);
		graphData.lpfnGetData      = (LPFNGetDataCallBack)      MakeProcInstance((FARPROC) LOCAL_GetDataCallback,   hInst);
		graphData.lpfnSetPage      = (LPFNSetPageCallBack)      MakeProcInstance((FARPROC) LOCAL_SetPageCallback,   hInst);
		graphData.lpfnRealToString = (LPFNRealToStringCallBack) MakeProcInstance((FARPROC) LOCAL_RealToStrCallback, hInst);
	   }
   
	/*
	** always set datalook-related info
	*/
	graphData.nPages      = 1;	 	/* THIS SHOULD ALWAYS BE SET TO 1 */
	graphData.nRows       = 3;		/* SET TO NUMBER OF ROWS TO GRAPH */
	graphData.nCols       = 3;		/* SET TO NUMBER OF COLUMNS TO GRAPH */
		/* PUT HANDLE TO THIS GRAPH INTO A FIELD THAT IS PASSED TO EVERY
		   DATA CALLBACK FUNCTION.  THIS LETS THE FUNCTION KNOW WHICH GRAPH
			IS DOING THE CALLING. */
	graphData.lClientData = (INT32) pGraph;	

	/* 
	** PG - Register the callback functions with the PG library 
	*/
	return SetGraphDataInfo(pGraph, &graphData);
}


/************************************************************************
* 	FiniDataCallBacks
*
*	DESCRIPTION:
*		This function dis-connects the above CallBack functions with a 
*		particular graph.
*		
*	RETURNS:
*		nothing.
*
************************************************************************/
VOID PUBLIC FiniDataCallBacks(GraphPtr pGraph)
{
	GraphDataInfo		graphData;
	
	GetGraphDataInfo(pGraph, &graphData);

	if (graphData.lpfnGetString) 
	  {
		FreeProcInstance( graphData.lpfnGetString    );
		FreeProcInstance( graphData.lpfnGetLabel     );
		FreeProcInstance( graphData.lpfnGetData      );
		FreeProcInstance( graphData.lpfnSetPage      );
		FreeProcInstance( graphData.lpfnRealToString );
	  }
	
}

void NEAR PASCAL UpdateMenu (HMENU hmenu, WORD *mVar, WORD wParam)
	{
	CheckMenuItem(hmenu,*mVar,MF_BYCOMMAND | MF_UNCHECKED);
	*mVar = wParam;
	CheckMenuItem(hmenu,*mVar,MF_BYCOMMAND | MF_CHECKED);
	}

