/*--------------------------------------------------------------------------*\
  FILE: SAMP17S.C                                            

  DESCIPTION:
    This is a sample program that demonstrates the Send Data interface
    version of Presentation Graphics.

  WRITTEN BY: Dan Weingart
  DATE: September 21, 1991

	MODIFIED BY: Terry Lahman
	DATE: May 9, 1994
	DATE: March 30, 1994

  COMPILER: Microsoft C++

  OTHER FILES NEEDED TO COMPILE:
    SAMP17S.MAK
    SAMP17S.H
    SAMP17S.DEF
    pg.H
    pg.LIB

    DLGOPEN.H
    DLGOPEN.LIB

    SAMP17S.RC      * RESOURCE FILE *
    TDG_LOGO.ICO  * RESOURCE FILE *

  OTHER FILES NEEDED TO RUN:
		default.tdc
    pg.DLL
\*--------------------------------------------------------------------------*/

#include <Windows.H>
#include <stdlib.h>
#include "SAMP17S.H"
#include "pg.h"	/* PRESENTATION GRAPHICS SDK LIBRARY */
#include "dlgopen.h"						/* needed for OpenFile dialog */

int FAR              dprintf(char FAR *fmt, ...);

/*--------------------------------------------------------------------------*\
|                        FUNCTION PROTOTYPES                                |
\*--------------------------------------------------------------------------*/

long  FAR   PASCAL  RecWndProc (HWND, WORD, WORD, LONG);

/* PROGRAM SPECIFIC CALLS TO HOOK UP PG LIBRARY TO LOCAL DATA SOURCES */
BOOL PUBLIC InitDataCallBacks(HANDLE hInst, GraphPtr pGraph);
VOID PUBLIC FiniDataCallBacks(GraphPtr pGraph);


int  NEAR  DoReadFile(HWND hWnd,GraphPtr  pGraph,char FAR *pName, INT16 nType);
int NEAR SetWinTitle(HWND  hWnd, char FAR *pName);

int  NEAR  DoWriteFile(HWND hWnd,GraphPtr pGraph,char FAR *pName,int nType);


int  NEAR  GetInputFileName(HWND hWnd, char FAR *pFileName,INT16  nFileType);


int  NEAR  GetOutputFileName(HWND hWnd,char FAR *pFileName,INT16  nFileType);


VOID PUBLIC UpdateBitmap(HDC hdc, RECT *r);

VOID	NEAR	PASCAL	ChangeGraphType (WORD);
VOID	NEAR	PASCAL	UpdateMenu  (HMENU hmenu, WORD *mVar, WORD wParam);

VOID PUBLIC NewGraphAll(GraphPtr);
VOID PUBLIC NewGraphStrings(GraphPtr pGraph, char *lpStingData);
VOID PUBLIC NewGraphSeriesLabels(GraphPtr pGraph, int nRows, char *lpStringLabel);
VOID PUBLIC NewGraphGroupsLabels(GraphPtr pGraph, int nCols, char *lpStringLabel);
VOID PUBLIC NewGraphData(GraphPtr pGraph, int nRows, int nCols, int *lpNumData);
INT16 NEAR XOR_List (HWND hWnd, SelListPtr pSelList, DrawEnvPtr pDrawEnv);
/*--------------------------------------------------------------------------*\
|                       Static Data.                                       |
\*--------------------------------------------------------------------------*/
HANDLE  hInst;

/* PG - Global graph variables */
SelItemPtr		gpSelItem;						/* ptr to Selection Item */
SelListPtr		gpList;							/* ptr to Selection List */
DetNodeRef		gDetNode;						/* global DetNode			 */
DetNodeRefPtr	gpDetNode = &gDetNode;		/* ptr to global DetNode */
DetNodePtr		tempNode;						/* ptr to global DetNode */
DrawEnvPtr 		gpDrawEnv;             		/* ptr to StdDraw Environment */
GraphPtr   		gpGraph;                  /* ptr to Graph Object */
int        		gbGraphValid;             /* TRUE when graph exists */
char       		gFileName[256];
LOGFONT   		lfControl;
LOGFONT   		lfData;
HFONT     		hfControl;
HFONT     		hfData;
BOOL  				bTRUE = TRUE;
BOOL					bFALSE= FALSE;
int						nDataSource = 1;					/* 1 - use data source 1
																				** 2 - use data source 2
																				*/

// TL:10/19/92 hdcMem, oldRect, and hBitmap to test drawing to a memory DC
// and then BitBlt to the screen display DC
HDC			hdcMem=0;					// memory context for the bitmap
RECT		r,rOld;
RECT		oldRect;					// the rectangle for drawing in the window
HBITMAP	hBitmap=0;				// handle to a memory bitmap
HBITMAP hOldBitmap=0;			// handle to the original bitmap (usually null)
#define dcDISPLAY  1			// normal display dc
#define dcBITMAP   2			// display to a memory dc (bitmap)
#define dcMETAFILE 3			// display to a metafile
INT16		bPortMode = DE_PORT_NORMAL;	// port DC used to draw on
BOOLEAN bRedraw = 0;			// set when the operator selects redraw on menu bar
BOOLEAN16	gbDirection;


/* THE FOLLOWING STATIC VARIABLES ARE USED TO STORE THE CHART DATA */
#define MaxStringSz		64
#define DATA1ROWS			4
#define DATA1COLS			3
#define DATA2ROWS			6
#define DATA2COLS			5

	TDG_ScaleValuesInfo thisManual = {TRUE,
					1000.0,					/* upper limit of the scale */
					400.0};					/* lower limit of the scale */

int   NumData1 [DATA1ROWS][DATA1COLS] = 
								{{200,300,400},
								{300,400,500},
								{400,500,600},
								{500,600,700}};

int   NumData2 [DATA2ROWS][DATA2COLS] =
												{{300,400,500,600,700},
											  {430,520,610,720,880},
											  {460,540,630,740,860},
											  {490,560,650,760,840},
											  {600,580,400,300,200},
											  {420,510,620,710,820}};

char *StringData;

char	StringData1[31][MaxStringSz] = {"PG Sample Chart",
												 "by Three |D| Graphics",
												 "Dan", "Elmer","Jon","Keith",
												 "Martin","Kai","Neil","Wayne",
												 "James","Bryan","Sam","Celia",
												 "January", "February", "March","April",
												 "May","June","July","August",
												 "September","October","November","December",
												 "The Row Title", "The Column Title",
												 "A footnote", "Y1 Title", "Y2 Title"};

char	StringData2[31][MaxStringSz] = {"PG Sample Chart 2",
												 "by Three |D| Graphics 2",
												 "Dan 2", "Elmer 2","Jon 2","Keith 2",
												 "Martin 2","Kai 2","Neil 2","Wayne 2",
												 "James 2","Bryan 2","Sam 2","Celia 2",
												 "January 2", "February 2", "March 2","April 2",
												 "May 2","June 2","July 2","August 2",
												 "September 2","October 2","November 2","December 2",
												 "The Row Title 2", "The Column Title 2",
												 "A footnote 2", "Y1 Title 2", "Y2 Title 2"};

char FAR *FileTypeExt[] = {
  "*.TDC",
  "*.TDL",
  "*.BMP",
  "*.WMF",
  "*.XLS",				/* IMPORT ADDITION */
  "*.WK1",				/* IMPORT ADDITION */
  "*.DIF",
  "*.DBF",
  "*.TXT"
};

enum FileType {
  FILE_TDC,
  FILE_LOOK,
  FILE_BMP,
  FILE_WMF,
  FILE_XLS,				/* IMPORT ADDITION */
  FILE_WK1,				/* IMPORT ADDITION */
  FILE_DIF,
  FILE_DBF,
  FILE_TXT,
  FILE_LOOKBACKGROUND,
  FILE_LOOKGRAPH,
  FILE_LOOKANNOTATION,
  MAX_FileType
};

//CHAR FileFilter[] = "TDL Files (*.TDL)\0*.TDL\03DL Files (*.3DL)\0*.3DL\0WMF Files (*.WMF)\0*.WMF\0";
CHAR FileFilter[] = "TDL Files (*.TDL)\0*.TDL\0";
CHAR TDCFileFilter[] = "TDC Files (*.TDC)\0*.TDC\0";


/*--------------------------------------------------------------------------*\
|                       Main Function: WinMain                             |
\*--------------------------------------------------------------------------*/

int PASCAL WinMain  (HANDLE hInstance,
               HANDLE hPrevInstance,
               LPSTR  lpszCmdLine,
               int    cmdShow)

{
  HWND     hwnd;
  MSG      msg;
  WNDCLASS wndclass;
  INT16    nStatus;
  char     tmpbuf[256];

  if (!hPrevInstance)
  {
    wndclass.lpszClassName = "SAMP17S:MAIN";
    wndclass.hInstance     = hInstance;
    wndclass.lpfnWndProc   = RecWndProc;
    wndclass.hCursor       = LoadCursor (NULL, IDC_ARROW);
    wndclass.hIcon         = LoadIcon (hInstance, "TDG_LOGO");
    wndclass.lpszMenuName  = "SAMP17S_MENU";
    wndclass.hbrBackground = COLOR_WINDOW+1;
    wndclass.style         = CS_HREDRAW | CS_VREDRAW;
    wndclass.cbClsExtra    = 0;
    wndclass.cbWndExtra    = 0;

    RegisterClass( &wndclass);
  }
	StringData = StringData1[0];

  hInst = hInstance;

  if ( nStatus = InitTDGLIB() )     /* PG - MAIN INITIALIZATION CALL */
  {
    wsprintf(tmpbuf, "InitTDGLIB failed (%d)!", nStatus);
    MessageBox(NULL, tmpbuf, "WinMain",
            MB_OK | MB_ICONHAND | MB_SYSTEMMODAL );
  }

  if ( !(gpGraph = AllocGraphPtr()) ) /* PG - ALLOCATE A GRAPH POINTER */
    return FALSE;
            
                          /* PG - ALLOCATE DRAW ENVIRONMENT */
  if ( !(gpDrawEnv = AllocDrawEnvPtr(hwnd, FALSE)) )  
    return FALSE;

	InitDataCallBacks(hInst, gpGraph);  

	if (nStatus != E00_OK)
	 	return FALSE;
								  /* PG SDK- CREATE AN EMPTY SELECTION LIST */
  gpList = Select_AllocList();

  hwnd = CreateWindow("SAMP17S:MAIN",     /* Class Name. */
                "PG Sample #17S",       /* Title      */
                WS_OVERLAPPEDWINDOW,    /* Style bits. */
                CW_USEDEFAULT,          /* x - default */
                0,                      /* y - default  */
                CW_USEDEFAULT,          /* cx - default*/
                0,                      /* cy - default*/
                NULL,                   /* No parent  */
                NULL,                   /* Class menu. */
                hInstance,              /* Creator    */
                NULL                    /* Params    */
               ) ;

	/* PG- LOAD A BAR GRAPH TEMPLATE AS DEFAULT */
	nStatus = TDIReadGraph(gpGraph, REQUEST_TDC, "default.tdc");

	GetClientRect(hwnd, &r);
	rOld = r;
	gbGraphValid = TRUE;

	SetWinTitle(hwnd, "default.tdc");

  ShowWindow (hwnd, cmdShow);

  while (GetMessage (&msg, 0, 0, 0))
  {
    TranslateMessage (&msg);    /* Keyboard input. */
    DispatchMessage (&msg);
  }


  /* PERFORM PG CLEAN-UP */

  Select_FreeList (gpList);		 /* PG SDK - FREE MEMORY USED BY THIS POINTER */

  FreeDrawEnvPtr(gpDrawEnv);  	/* PG - FREE MEMORY USED BY THIS POINTER */

  FreeGraphPtr(gpGraph);      	/* PG - FREE MEMORY USED BY THIS POINTER */

  FiniTDGLIB();

  return 0;
}

/*--------------------------------------------------------------------------*\
|                       Window Procedure: RecWndProc.                      |
|   This is the main message processing loop for this application           |
\*--------------------------------------------------------------------------*/

long FAR PASCAL RecWndProc(HWND hWnd, WORD msg, WORD wParam, LONG lParam)
{
  static BOOL  fCapture = FALSE;
  static BOOL  fDrag    = FALSE;
  static POINT ptDrag;
  HMENU        hmenu;
  char         buffer[80];
  static int   one = 1;

  static WORD wGraphID = IDM_BAR;

  switch (msg)
  {
    case WM_COMMAND:
      hmenu = GetMenu (hWnd);
      switch (wParam)
      {
        case IDM_QUIT:         /* TIME TO QUIT! */
				  DestroyWindow (hWnd);
				  break;
        
        case MFILE_OPEN:
          if ( GetInputFileName(hWnd, gFileName, FILE_TDC) )
          {
	          gbGraphValid = (DoReadFile(hWnd, gpGraph, gFileName, FILE_TDC) == E00_OK);
						SetWinTitle(hWnd, gFileName);
		        InvalidateRect (hWnd, &rOld, TRUE);
					}
          break;

        case MFILE_OPEN_LOOK:
			  	if( GetInputFileName(hWnd, gFileName, FILE_LOOK) )
					  gbGraphValid = (DoReadFile(hWnd, gpGraph, gFileName, FILE_LOOK) == E00_OK);
					if(gbGraphValid)
					 	SetWinTitle(hWnd, gFileName);
				  InvalidateRect (hWnd, &rOld, TRUE);
					break;

        case MFILE_OPEN_LOOKGRAPH:
			  	if( GetInputFileName(hWnd, gFileName, FILE_LOOK) )
					  gbGraphValid = (DoReadFile(hWnd, gpGraph, gFileName,
															FILE_LOOKGRAPH) == E00_OK);
					if (gbGraphValid)
					 	SetWinTitle(hWnd, gFileName);
				  InvalidateRect (hWnd, &rOld, TRUE);
					break;

        case MFILE_OPEN_LOOKBACKGROUND:
			  	if( GetInputFileName(hWnd, gFileName, FILE_LOOK) )
					  gbGraphValid = (DoReadFile(hWnd, gpGraph, gFileName,
															FILE_LOOKBACKGROUND) == E00_OK);
					if(gbGraphValid)
					 	SetWinTitle(hWnd, gFileName);
				  InvalidateRect (hWnd, &rOld, TRUE);
					break;

        case MFILE_SAVE_LOOK:
			  	if( GetOutputFileName(hWnd, gFileName, FILE_LOOK) )
					  DoWriteFile(hWnd, gpGraph, gFileName, FILE_LOOK);
					break;

        case MFILE_SAVE_LOOKBACKGROUND:
			  	if( GetOutputFileName(hWnd, gFileName, FILE_LOOK) )
					  DoWriteFile(hWnd, gpGraph, gFileName, FILE_LOOKBACKGROUND);
					break;

        case MFILE_SAVE_LOOKGRAPH:
			  	if( GetOutputFileName(hWnd, gFileName, FILE_LOOK) )
					  DoWriteFile(hWnd, gpGraph, gFileName, FILE_LOOKGRAPH);
					break;

			  case MFILE_SAVE:
			  	if( GetOutputFileName(hWnd, gFileName, FILE_TDC) )
	          DoWriteFile(hWnd, gpGraph, gFileName, FILE_TDC);
          break;
  

				case IDM_BAR:
      	case IDM_LINE:
      	case IDM_AREA:
      	case IDM_3D:
      	case IDM_PIE:
  	    case IDM_BARS:
    	  case IDM_BARP:
				case IDM_SCATTER:

			  	UpdateMenu (hmenu,&wGraphID,wParam);
				  ChangeGraphType (wGraphID);	/* SEE SUB-ROUTINE FOR PG CALL */
          InvalidateRect (hWnd, &rOld, TRUE);
				  break;

        case IDM_REDRAW:
					bRedraw = TRUE;
          InvalidateRect (hWnd, &rOld, TRUE);
          UpdateWindow(hWnd);           /* Sends WM_PAINT message */
	        break;

				case IDM_NEWDATA_GRAPH:
					NewGraphAll(gpGraph);
					InvalidateRect (hWnd, &rOld, TRUE);
					break;

				case IDM_NEWDATA_STRINGS:
					NewGraphStrings(gpGraph, 
													nDataSource == 1 ? StringData2[0] : StringData1[0]);
					InvalidateRect (hWnd, &rOld, TRUE);
					break;

				case IDM_NEWDATA_TITLE:
					SetGraphTitle(gpGraph,"New Title");
					InvalidateRect (hWnd, &rOld, TRUE);
					break;

				case IDM_NEWDATA_SUBTITLE:
					SetGraphSubTitle(gpGraph,"New Sub Title");
					InvalidateRect (hWnd, &rOld, TRUE);
					break;

				case IDM_NEWDATA_FOOTNOTE:
					SetGraphFootNote(gpGraph,"New Footnote");
					InvalidateRect (hWnd, &rOld, TRUE);
					break;

				case IDM_NEWDATA_SERIESTITLE:
					SetGraphSeriesTitle(gpGraph,"New Series Title");
					InvalidateRect (hWnd, &rOld, TRUE);
					break;

				case IDM_NEWDATA_GROUPSTITLE:
					SetGraphGroupsTitle(gpGraph,"New Groups Title");
					InvalidateRect (hWnd, &rOld, TRUE);
					break;

				case IDM_NEWDATA_Y1TITLE:
					SetGraphY1AxisTitle(gpGraph,"New Y1 Axis Title");
					InvalidateRect (hWnd, &rOld, TRUE);
					break;

				case IDM_NEWDATA_Y2TITLE:
					SetGraphY2AxisTitle(gpGraph,"New Y2 Axis Title");
					InvalidateRect (hWnd, &rOld, TRUE);
					break;

				case IDM_NEWDATA_LABELS:
					NewGraphSeriesLabels(gpGraph, 
								nDataSource == 1 ? DATA2ROWS : DATA1ROWS,
								nDataSource == 1 ? StringData2[2] : StringData1[2]);
					NewGraphGroupsLabels(gpGraph, 
								nDataSource == 1 ? DATA2COLS : DATA1COLS,
								nDataSource == 1 ? StringData2[14] : StringData1[14]);
					StringData = StringData==StringData1[0]?StringData2[0]:StringData1[0];
					InvalidateRect (hWnd, &rOld, TRUE);
					break;

				case IDM_NEWDATA_SERIES1:
					SetGraphSeriesLabel(gpGraph,0,"New Series 1 Label");
					InvalidateRect (hWnd, &rOld, TRUE);
					break;

				case IDM_NEWDATA_GROUPS1:
					SetGraphGroupsLabel(gpGraph,0,"New Group 1 Label");
					InvalidateRect (hWnd, &rOld, TRUE);
					break;

				case IDM_NEWDATA_DATACELL:
					SetGraphRowColData(gpGraph,0,0, 155.0);
					InvalidateRect (hWnd, &rOld, TRUE);
					break;

				case IDM_DCNORMAL:
		    	CheckMenuItem(hmenu,IDM_DCNORMAL, MF_CHECKED);
		    	CheckMenuItem(hmenu,IDM_DCBITMAP, MF_UNCHECKED);
					bPortMode = dcDISPLAY;
					bRedraw = FALSE;
					InvalidateRect (hWnd, &rOld, TRUE);
          UpdateWindow(hWnd);           /* Sends WM_PAINT message */
					break;

				case IDM_DCBITMAP:
		    	CheckMenuItem(hmenu,IDM_DCNORMAL, MF_UNCHECKED);
		    	CheckMenuItem(hmenu,IDM_DCBITMAP, MF_CHECKED);
					bPortMode = dcBITMAP;
					bRedraw = FALSE;
					InvalidateRect (hWnd, &rOld, TRUE);
          UpdateWindow(hWnd);           /* Sends WM_PAINT message */
					break;

				case IDM_Y1MANUAL:
					SetGraphAttrSI (gpGraph,NULL, A2D_SCALE_Y1, &thisManual);
					InvalidateRect (hWnd, &rOld, TRUE);
          UpdateWindow(hWnd);           /* Sends WM_PAINT message */
					break;

				case IDM_Y1INVERT:
					GetGraphAttrSI(gpGraph,NULL,A2D_DIRECTION_Y1,&gbDirection);
					gbDirection = !gbDirection;
					SetGraphAttrSI(gpGraph,NULL,A2D_DIRECTION_Y1,&gbDirection);
					InvalidateRect (hWnd, &rOld, TRUE);
          UpdateWindow(hWnd);           /* Sends WM_PAINT message */
					break;

        default:
          wsprintf (buffer, "Command = %d", wParam);
          MessageBox (hWnd, buffer, "WM_COMMAND", MB_OK);
          break;

      }
      break;

    case WM_DESTROY:
			// TL:10/20/92 I need to eliminate the bitmap and memory dc if used
			if(hBitmap)
			{
				SelectObject(hdcMem, hOldBitmap);	// deselect the drawing bitmap
				DeleteObject(hBitmap);						// and delete the drawing bitmap
				hBitmap=0;
			}
			if(hdcMem)
				DeleteDC(hdcMem);

      if( hfControl )
        DeleteObject( hfControl );
      if( hfData )
        DeleteObject( hfData );
      PostQuitMessage (0);
      break;
    
    case WM_LBUTTONDOWN:
 			{
 			POINT pt;
 			POINTS pts;
			BOOLEAN bFoundItem;

 		  	pts = MAKEPOINTS (lParam);	/* Get the Mouse Pt. */
			pt.x = pts.x;
			pt.y = pts.y;
 		  	dvPoint(gpDrawEnv,&pt);		/*PG SDK - Convert to Virt. Coord's. */
			bFoundItem = FALSE;
   	   		XOR_List(hWnd,gpList,gpDrawEnv); 	 /*PG SDK - TURN XOR HANDLES ON */
	 	    Select_ClearList (gpList);  /*PG SDK - Clear our master list */	  
		    bFoundItem = FindNextDetNode (gpGraph, &pt, gpDetNode,bFoundItem);
			if (bFoundItem)
			 gpSelItem = Select_AddItem(gpGraph,gpList,gpDetNode);	  
   	      	Select_CalcHandlesList (gpDrawEnv,gpList);
   	  		XOR_List(hWnd,gpList,gpDrawEnv);			/*PG SDK - TURN XOR HANDLES ON */
		   }
			break;


    case WM_LBUTTONUP:
	    break;

    case WM_MOUSEMOVE:
      break;

    case WM_PAINT:
      {
      	PAINTSTRUCT ps;
	      HDC hdc;

				GetClientRect(hWnd, &r);

  	    hdc = BeginPaint (hWnd, &ps);

      	if (!fDrag)
        {
          if (gbGraphValid)
        	{
	        	DESetDrawProcs(gpDrawEnv, DE_PROCS_DEFAULT);

						// TL:10/20/92 if bitmap mode and the bitmap exist then bitblt
						if(bPortMode == dcBITMAP && hBitmap && hdcMem && bRedraw)
						{
							BitBlt(hdc, r.left, r.top, r.right - r.left, r.bottom - r.top, 
											hdcMem, 0, 0, SRCCOPY);
							EndPaint (hWnd, &ps);
							bRedraw = FALSE;
							return 0L;
							break;
						}
						bRedraw = FALSE;

						// TL:10/20/92 modified to support drawing to bitmaps
						if(bPortMode == dcDISPLAY)
							DESetPortInfo(gpDrawEnv, hdc, DE_PORT_NORMAL);
						else if(bPortMode == dcBITMAP)
						{
							UpdateBitmap(hdc,&r);
							if(hdcMem)
	      				DESetPortInfo(gpDrawEnv, hdcMem, DE_PORT_NORMAL);
							else
	      				DESetPortInfo(gpDrawEnv, hdc, DE_PORT_NORMAL);
						}

						/*PG - SET THE CHART'S SIZE & LOCATION IN WINDOW */
	          DESetDestRect(gpDrawEnv, &r);

//     /*PG - SET DRAWING MODE TO NORMAL, ON-SCREEN */
//          DESetPortInfo(gpDrawEnv, hdc, DE_PORT_NORMAL);

						/*PG - DRAW THE GRAPH */
          	DrawBegin(gpDrawEnv);
            	DrawTheGraph(gpDrawEnv, gpGraph, TRUE, TRUE, TRUE);
	          DrawEnd(gpDrawEnv);

						if(bPortMode == dcBITMAP)
						{
							RECT rswitch;
							rswitch = r;
							r=oldRect;
							oldRect=rswitch;
							if(hdcMem && hBitmap)		// make sure the bitmap really exists
							{
								BitBlt(hdc, r.left, r.top, r.right - r.left, r.bottom - r.top,
												hdcMem, 0, 0, SRCCOPY);
							}
						}

						/*PG - SET STANDARD DRAW BACK TO USING THE DEFAULT PROCS */
          	DESetDrawProcs(gpDrawEnv, DE_PROCS_DEFAULT);
		  /* PG SDK - Match up objects in selection list with new detnodes */
   		    Select_RebuildList (gpGraph,gpList);

		  /* PG SDK - Calculate handles for objects in selection list based on new detnodes*/
		    Select_CalcHandlesList (gpDrawEnv,gpList);

			}
        }

				EndPaint (hWnd, &ps);
			}
			break;

		case WM_SIZE:
			bRedraw = TRUE;
      break;

    default:
      return(DefWindowProc(hWnd,msg,wParam,lParam));
      break;
  }
  return 0L;
}

int NEAR SetWinTitle(HWND hWnd, char FAR *pName)
{
  char            tmpbuf[256];

  InvalidateRect(hWnd, NULL, TRUE);

  /* set the window title too */
  wsprintf(tmpbuf, "PG SAMPLE #17S - %s", pName);
  SetWindowText(hWnd, tmpbuf);

	return 0;
}

/************************************************************************
*	DoReadFile
*
*	DESCRIPTION:
*		Does a bunch of stuff related to reading a new 3Dx file.
*
*	RETURNS:
*
************************************************************************/
int NEAR DoReadFile(
	HWND						hWnd,
	GraphPtr				pGraph,
	char FAR *			pName,
	INT16						nType
)
{
	int							nResult;
	char						tmpbuf[256];

	SetCursor (LoadCursor (NULL, IDC_WAIT));
	
	/*
	 *  kill any existing DetNodes
	 */
	FreeAllDetNodes(pGraph);

	/*
	 *  kill the annotation stuff too
	 */
	FreeAllAnodes(pGraph);

	/*
	 *  clear the current selection list too
	 */
	switch (nType)
	{
		case FILE_TDC:			/*	This is usable only in the VDM version of PG */
	  	nResult = TDIReadGraph(pGraph, REQUEST_TDC, pName);
			break;

		case FILE_LOOK:
	  	nResult = TDIReadGraph(pGraph, REQUEST_TDL, pName);
			break;
	
		case FILE_LOOKBACKGROUND:
	  	nResult = TDIReadGraph(pGraph, REQUEST_TDLBACKGROUND, pName);
			break;
	
		case FILE_LOOKGRAPH:
	  	nResult = TDIReadGraph(pGraph, REQUEST_TDLGRAPH, pName);
			break;
	
		default:
			nResult = -1;
			break;
	}

	InvalidateRect(hWnd, NULL, TRUE);

	/* set the window title too */
	wsprintf(tmpbuf, "PG SAMPLE #17S - %s", pName);
	SetWindowText(hWnd, tmpbuf);

	SetCursor (LoadCursor (NULL, IDC_ARROW));
  
	/* return the error code */
	return nResult;
}

/************************************************************************
*	DoWriteFile
*
*	DESCRIPTION:
*		Does a bunch of stuff related to reading a new 3Dx file.
*
*	RETURNS:
*
************************************************************************/
int NEAR DoWriteFile(
	HWND						hWnd,
	GraphPtr				pGraph,
	char FAR *			pName,
	int							nType
)
{
	int							nResult;
	
	SetCursor (LoadCursor (NULL, IDC_WAIT));
	switch (nType)
	{
		case FILE_TDC:			/*  This is usable only in the VDM version of PG  */
		  nResult = TDIWriteGraph(pGraph, REQUEST_TDC, pName);
			break;

		case FILE_LOOK:	
		  nResult = TDIWriteGraph(pGraph, REQUEST_TDL, pName);
			break;

		case FILE_LOOKBACKGROUND:	
		  nResult = TDIWriteGraph(pGraph, REQUEST_TDLBACKGROUND, pName);
			break;

		case FILE_LOOKGRAPH:	
		  nResult = TDIWriteGraph(pGraph, REQUEST_TDLGRAPH, pName);
			break;

		default:
			nResult = -1;
			break;
	}
	SetCursor (LoadCursor (NULL, IDC_ARROW));
	
	return nResult;
}


/************************************************************************
*  GetInputFileName
*
*  DESCRIPTION:
*    Requests the user to choose an existing file.
*
*  RETURNS:
*    TRUE if user chose something
*
************************************************************************/
int NEAR
GetInputFileName(
  HWND            hWnd,
  char FAR *      pFileName,
  INT16            nFileType
)
{
  int              nResult;
  char            fName[256];
  OPENFILENAME	   OpenFileName;


  /*** clone filename to prevent damage ***/
  lstrcpy(fName, pFileName);
  

  /*** Call DlgOpen() to get the filename ***/

#if 0
  nResult = DlgOpenFile( hWnd,
                        "Select an Input File",
                        (LONG) OF_EXIST | OF_MUSTEXIST | OF_NOOPTIONS,
                        FileTypeExt[nFileType],
                        fName,
                        NULL);
#else
   OpenFileName.lStructSize       = sizeof(OPENFILENAME);
   OpenFileName.hwndOwner         = hWnd;
   OpenFileName.hInstance         = (HANDLE) hInst;
   if (nFileType == FILE_TDC)
   	 OpenFileName.lpstrFilter       = TDCFileFilter;
   else
   	 OpenFileName.lpstrFilter       = FileFilter;
   OpenFileName.lpstrCustomFilter = (LPSTR) NULL;
   OpenFileName.nMaxCustFilter    = 0L;
   OpenFileName.nFilterIndex      = 1L;
   OpenFileName.lpstrFile         = fName;
   OpenFileName.nMaxFile          = sizeof(fName);
   OpenFileName.lpstrFileTitle    = (LPSTR) NULL;
   OpenFileName.nMaxFileTitle     = 0L;
   OpenFileName.lpstrInitialDir   = NULL;
   OpenFileName.lpstrTitle        = "Select an Input File";
   OpenFileName.nFileOffset       = 0;
   OpenFileName.nFileExtension    = 0;
   OpenFileName.lpstrDefExt       = FileTypeExt[nFileType];
   OpenFileName.lCustData         = 0;
   OpenFileName.Flags = OFN_SHOWHELP | OFN_PATHMUSTEXIST |
       OFN_FILEMUSTEXIST;

	nResult = GetOpenFileName(&OpenFileName);
#endif

  if (nResult)
    lstrcpy(pFileName, fName);

  return (nResult > 0);
}


/************************************************************************
*   GetOutputFileName
*
*  DESCRIPTION:
*    Requests the user to choose an existing file.
*
*  RETURNS:
*    TRUE if user chose something
*
************************************************************************/
int NEAR
GetOutputFileName(
  HWND            hWnd,
  char FAR *      pFileName,
  INT16            nFileType
)
{
  int              nResult;
  char            fName[256];
	OPENFILENAME				OpenFileName;

  /*** clone filename to prevent damage ***/
  lstrcpy(fName, pFileName);
  
  /* Call DlgOpen() to set the filename */
  
#if 0  
  nResult = DlgOpenFile( hWnd,
                        "Enter Output Filename",
                        (LONG) OF_SAVE | OF_SHOWSPEC | OF_NOOPTIONS,
                        FileTypeExt[nFileType],
                        fName,
                        NULL);

#else
   OpenFileName.lStructSize       = sizeof(OPENFILENAME);
   OpenFileName.hwndOwner         = hWnd;
   OpenFileName.hInstance         = (HANDLE) hInst;
   OpenFileName.lpstrFilter       = (LPSTR) NULL;
   OpenFileName.lpstrCustomFilter = (LPSTR) NULL;
   OpenFileName.nMaxCustFilter    = 0L;
   OpenFileName.nFilterIndex      = 0L;
   OpenFileName.lpstrFile         = fName;
   OpenFileName.nMaxFile          = sizeof(fName);
   OpenFileName.lpstrFileTitle    = (LPSTR) NULL;
   OpenFileName.nMaxFileTitle     = 0L;
   OpenFileName.lpstrInitialDir   = NULL;
   OpenFileName.lpstrTitle        = "Enter Output Filename";
   OpenFileName.nFileOffset       = 0;
   OpenFileName.nFileExtension    = 0;
   OpenFileName.lpstrDefExt       = FileTypeExt[nFileType];
   OpenFileName.lCustData         = 0;
   OpenFileName.Flags = 0L;

	nResult = GetSaveFileName(&OpenFileName);
#endif
    
  if (nResult)
    lstrcpy(pFileName, fName);

	return  1; /* (nResult >= 0);*/
}


/*--------------------------------------------------------------------------*\
|          	  39ChangeGraphType: Call PG change graph type routine. 		    |
| This routine takes a single parameter, graphID, and uses its value to 	 |
| determine an index into an array of type TDG_GraphPresetInfo. Each element|
| in this array represents the 2 parameters needed by the PG call			 |
| SetGraphAttrSL needed to select a preset graphtype. 							 |
\*--------------------------------------------------------------------------*/
void NEAR PASCAL ChangeGraphType (WORD graphID)
{
  int iGraph;
	static	TDG_GridLinesOrdInfo thisGridLine = { 0,	/* no major grid lines */
																		0,		/* no minor grid lines */
																		0,		/* minor hash count */
																		0,		/* no super grid lines */
																		0 };	/* super grid disabled */

  static TDG_GraphPresetInfo GraphTable [12] = {{TDG_GRAPH_BARS , 8},
															  {TDG_GRAPH_LINES, 0},
															  {TDG_GRAPH_AREA , 4},
															  {TDG_GRAPH_3D   , 0},
															  {TDG_GRAPH_PIE,   0},
															  {TDG_GRAPH_BARS,  4},
															  {TDG_GRAPH_BARS,  12},
																{TDG_GRAPH_SPECIAL, 0},
																{TDG_GRAPH_BARS,  6},
																{TDG_GRAPH_BARS,  7},
															  {TDG_GRAPH_PIE,   6},
																{TDG_GRAPH_SPECIAL, 3} };
	int two = 2;
	int zero = 0;
	int four = 4;

	// make sure the number of cells for a node is one
	SetGraphAttrSL(gpGraph,NULL,A2D_DATAFORMAT,&zero);
  switch (graphID)
	{
	  case IDM_BAR:
 	    SetGraphAttrSI (gpGraph, NULL, A2D_EXCLUDE_ZERO_Y1,&bFALSE);
	    iGraph = 0;
		  break;
	  case IDM_LINE:
 	    SetGraphAttrSI (gpGraph, NULL, A2D_EXCLUDE_ZERO_Y1,&bTRUE);
	    iGraph = 1;
			break;
		case IDM_AREA:
 	  	SetGraphAttrSI (gpGraph, NULL, A2D_EXCLUDE_ZERO_Y1,&bTRUE);
	  	iGraph = 2;
			break;
		case IDM_3D:
	  	iGraph = 3;
			break;
		case IDM_PIE:
	  	iGraph = 4;
			break;
		case IDM_PIEBAR:
			/* turn off some of the things that clutter up the bar portion of the pie bar */
			SetGraphAttrSI (gpGraph,NULL, A2D_GRIDLINESORD_O1, &thisGridLine);
			SetGraphAttrSI (gpGraph,NULL, A2D_SHOW_X, &bFALSE);
			SetGraphAttrSI (gpGraph,NULL, A2D_SHOW_Y1, &bFALSE);
			SetGraphAttrSI (gpGraph,NULL, A2D_SIDE_O1, &four);
			iGraph = 10;
			break;
		case IDM_HISTOGRAM:
			iGraph = 7;
			break;
		case IDM_BARS:
 	  	SetGraphAttrSI (gpGraph, NULL, A2D_EXCLUDE_ZERO_Y1,&bFALSE);
	  	iGraph = 5;
			break;
		case IDM_BARP:
 	  	SetGraphAttrSI (gpGraph, NULL, A2D_EXCLUDE_ZERO_Y1,&bFALSE);
	  	iGraph = 6;
			break;
		case IDM_BARSB:
	  	iGraph = 8;
			break;
		case IDM_BARSDB:
	  	iGraph = 9;
			break;
		case IDM_SCATTER:
			// change the data format for X,Y,LABELS
  	  SetGraphAttrSL(gpGraph,NULL,A2D_DATAFORMAT,&two);
			iGraph = 11;
			break;
	}
  /* PG - MAKE CALL TO SELECT A DIFFERENT PRESET GRAPH TYPE */
  SetGraphAttrSL (gpGraph,NULL,A_GRAPH_PRESET, &GraphTable[iGraph]);
// next line to help test effects to horizontal graphs
//  SetGraphAttrSL(gpGraph,NULL,A2D_ORIENTATION,&bFALSE);
}

/*--------------------------------------------------------------------------*\
|            	  UpdateMenu: Update Checkmarks on main menu               	 |
\*--------------------------------------------------------------------------*/

void NEAR PASCAL UpdateMenu (HMENU hmenu, WORD *mVar, WORD wParam)
{
	CheckMenuItem(hmenu,*mVar,MF_BYCOMMAND | MF_UNCHECKED);
	*mVar = wParam;
	CheckMenuItem(hmenu,*mVar,MF_BYCOMMAND | MF_CHECKED);
}

/************************************************************************
* 	InitDataCallBacks
*
*	DESCRIPTION:
*		This function connects the above CallBack functions with a 
*		particular graph.
*		
*	RETURNS:
*		FALSE if any of the callback functions are set to NULL.
*
************************************************************************/

BOOL PUBLIC InitDataCallBacks(HANDLE hInst, GraphPtr pGraph)
{
	GraphDataInfo		graphData;
	
	/*
	** PG - Look up Data information connected to this graph
   */
	GetGraphDataInfo(pGraph, &graphData);
	
	/*
	** always set datalook-related info
	*/
	graphData.nPages      = 1;	 	/* THIS SHOULD ALWAYS BE SET TO 1 */
	graphData.nRows       = DATA1ROWS;		/* SET TO NUMBER OF ROWS TO GRAPH */
	graphData.nCols       = DATA1COLS;		/* SET TO NUMBER OF COLUMNS TO GRAPH */
		/* PUT HANDLE TO THIS GRAPH INTO A FIELD THAT IS PASSED TO EVERY
		   DATA CALLBACK FUNCTION.  THIS LETS THE FUNCTION KNOW WHICH GRAPH
			IS DOING THE CALLING. */
	graphData.lClientData = (INT32) pGraph;
	graphData.nDataIFMode = DATAIF_SENDDATA;

/* To be safe, we always set unused callbacks to NULL.  Since this sample uses
   the SEND_DATA/DM method, ALL of the callbacks should be set to NULL */

	graphData.lpfnGetString    = NULL;
	graphData.lpfnGetLabel     = NULL;
	graphData.lpfnGetData      = NULL;
	graphData.lpfnSetPage      = NULL;
	graphData.lpfnRealToString = NULL;
	graphData.lpfnGetRawLimits = NULL; /* If not using MUST set to NULL */
	graphData.lpfnPicture      = NULL; /* If not using MUST set to NULL */

	/* 
	** PG - Register the callback functions with the PG library 
	*/
	return SetGraphDataInfo(pGraph, &graphData);
}


/************************************************************************
* 	FiniDataCallBacks
*
*	DESCRIPTION:
*		This function dis-connects the above CallBack functions with a 
*		particular graph.
*		
*	RETURNS:
*		nothing.
*
************************************************************************/
VOID PUBLIC FiniDataCallBacks(GraphPtr pGraph)
{
	GraphDataInfo		graphData;
	
	GetGraphDataInfo(pGraph, &graphData);

}

VOID PUBLIC UpdateBitmap(HDC hdc, RECT *r)
{
	oldRect = *r;	// save a copy of the old rectangle
	// does the memory dc and the bitmap already exist, if so, then delete
	if(hBitmap)
	{
		SelectObject(hdcMem, hOldBitmap);	// deselect the drawing bitmap
		DeleteObject(hBitmap);						// and delete the drawing bitmap
		hBitmap=0;
	}
	if(hdcMem)
		DeleteDC(hdcMem);

	// create a memory device context and a compatible bitmap
	hdcMem = CreateCompatibleDC(hdc);
	if(hdcMem)			// make sure the memory dc handle is ok
	{
		hBitmap = CreateCompatibleBitmap(hdc, r->right - r->left, r->bottom - r->top); 
		if(hBitmap)		// make sure the bitmap created ok
		{
			// change the drawing rectangle to one for the bitmap
			r->left = r->top = 0;
			r->right = oldRect.right - oldRect.left;
			r->bottom = oldRect.bottom - oldRect.top;
			// select the bitmap into the memory device context so I have 
			// a surface to actually draw on
			hOldBitmap = SelectObject(hdcMem, hBitmap);
			SetMapMode(hdcMem, GetMapMode(hdc));
			// now tell the draw routines to use the memory device context
		}
		else				// couldn't create the bitmap so the mem dc is of no use
		{
			DeleteDC(hdcMem);
			hdcMem=0;
		}
	}
}

VOID PUBLIC NewGraphAll(GraphPtr pGraph)
{
	GraphDataInfo		graphData;
	int		*lpNumData;
	char	*lpStringData;
	
	/*
	** PG - Look up Data information connected to this graph
   */
	GetGraphDataInfo(pGraph, &graphData);

	if(nDataSource == 1)
	{
		graphData.nRows = DATA2ROWS;		/* SET TO NUMBER OF ROWS TO GRAPH */
		graphData.nCols = DATA2COLS;		/* SET TO NUMBER OF COLUMNS TO GRAPH */
		lpNumData = &NumData2[0][0];
		lpStringData = StringData2[0];
		nDataSource = 2;
	}
	else
	{
		graphData.nRows = DATA1ROWS;		/* SET TO NUMBER OF ROWS TO GRAPH */
		graphData.nCols = DATA1COLS;		/* SET TO NUMBER OF COLUMNS TO GRAPH */
		lpNumData = &NumData1[0][0];
		lpStringData = StringData1[0];
		nDataSource = 1;
	}

	SetGraphDataInfo(pGraph, &graphData);

	NewGraphStrings(pGraph, lpStringData);
	NewGraphSeriesLabels(pGraph, graphData.nRows, lpStringData+(MaxStringSz*2));
	NewGraphGroupsLabels(pGraph, graphData.nCols, lpStringData+(MaxStringSz*14));
	NewGraphData(pGraph, graphData.nRows, graphData.nCols, lpNumData);

	/* load the new chart data into the Virtual Data Manager (VDM) */

	return;
}

VOID PUBLIC NewGraphStrings(GraphPtr pGraph, char *lpStringData)
{
	SetGraphTitle(gpGraph, 			(char FAR *)lpStringData+(MaxStringSz*0));
	SetGraphSubTitle(gpGraph,		(char FAR *)lpStringData+(MaxStringSz*1));
	SetGraphFootNote(gpGraph,		(char FAR *)lpStringData+(MaxStringSz*8));
	SetGraphSeriesTitle(gpGraph,(char FAR *)lpStringData+(MaxStringSz*26));
	SetGraphGroupsTitle(gpGraph,(char FAR *)lpStringData+(MaxStringSz*27));
	SetGraphY1AxisTitle(gpGraph,(char FAR *)lpStringData+(MaxStringSz*29));
	SetGraphY2AxisTitle(gpGraph,(char FAR *)lpStringData+(MaxStringSz*30));
}

VOID PUBLIC NewGraphSeriesLabels(GraphPtr pGraph, int nRows, char *lpStringLabel)
{
	int nLoop;		/* loop counter */

	for(nLoop = 0; nLoop < nRows; nLoop++, lpStringLabel += MaxStringSz)
	{
		SetGraphSeriesLabel(pGraph, nLoop, (char FAR *)lpStringLabel);
	}
}

VOID PUBLIC NewGraphGroupsLabels(GraphPtr pGraph, int nCols, char *lpStringLabel)
{
	int nLoop;		/* loop counter */

	for(nLoop = 0; nLoop < nCols; nLoop++, lpStringLabel += MaxStringSz)
	{
		SetGraphGroupsLabel(pGraph, nLoop, (char FAR *)lpStringLabel);
	}
}

VOID PUBLIC NewGraphData(GraphPtr pGraph, int nRows, int nCols, int *lpNumData)
{
	int	nRowLoop, nColLoop;

	for(nRowLoop = 0; nRowLoop < nRows; nRowLoop++)
	{
		for(nColLoop = 0; nColLoop < nCols; nColLoop++, lpNumData++)
		{
			SetGraphRowColData(pGraph, nRowLoop, nColLoop, (REAL64) *lpNumData);
		}
	}
}


/*--------------------------------------------------------------------------*\
|            	  XOR_List
\*--------------------------------------------------------------------------*/

INT16 NEAR XOR_List (HWND hWnd, SelListPtr pSelList, DrawEnvPtr pDrawEnv)
	{
	HDC hdc;
	INT16 ret = 0;
	RECT  r2;
   SelItemPtr		pSelItem;

	hdc = GetDC (hWnd);
   DESetPortInfo(pDrawEnv, hdc, DE_PORT_NORMAL);
   DrawBegin(pDrawEnv);
		ret =  Select_DrawXorList(pDrawEnv, pSelList);
   DrawEnd(pDrawEnv);
	ReleaseDC (hWnd,hdc);

	return (ret);
	}
