/*--------------------------------------------------------------------------*\
	FILE: SAMP9.C  		**** 32-bit Windows Version ****																			

	DESCIPTION:
		This application demonstrates how to create & manage PG annotations
		items (lines, boxes, etc).

	   The calls demonstrated here are:

		    Select_AllocList()
				Select_FreeList()
	     	dvPoint()
		    dvRect()	
				FindDetNode() 
				Select_CalcHandlesItem()
				Select_ClearList()
				Select_RebuildList()
				Select_CalcHandlesList()
				ExecGraphFuncSL()
		   	Remove()
				AllocPolyHandle()
		    AddPolyPoint()
		    ClosePolyHandle ()
		    CreateAnode()
		  	MoveSelectedItem()

		See the file SAMP1.DOC for more detailed information on how to use this
		program and the steps taken to add PG calls to applications in general.

	WRITTEN BY: Dan Weingart

	DATE: December 18, 1991
	UPDATED: March 3rd, 1995

	COMPILER: Visual C 2.0
	PG VERSION: PG32 (3/3/95)

	OTHER FILES NEEDED TO COMPILE:
		SAMP9.MAK
		SAMP9.H
		SAMP9.DEF
		PG.H
		PG.LIB

		SAMP9.RC			* RESOURCE FILE *
		TDG_LOGO.ICO	* RESOURCE FILE *

	OTHER FILES NEEDED TO RUN:
		DEFAULT.TDL
		PG.DLL
\*--------------------------------------------------------------------------*/

#include <Windows.H>
#include <stdlib.h>
#include <string.h>
#include "SAMP9.H"


#ifndef _WINDOWS	/* MUST SPECIFY WHICH PLATFORM CHARTING HEADER FILE IS ON */
#define _WINDOWS
#endif

#include "pg.h"	/* PRESENTATION GRAPHICS SDK LIBRARY */

/*--------------------------------------------------------------------------*\
|            	         FUNCTION PROTOTYPES                           		 |
\*--------------------------------------------------------------------------*/

long	FAR	PASCAL	RecWndProc (HWND, WORD, WORD, LONG);

VOID NEAR PASCAL Rect2DrawNotRect (HWND, int, int, int, int);
VOID NEAR PASCAL UpdateMenu  (HMENU hmenu, WORD *mVar, WORD wParam);

int	NEAR  DoReadFile(HWND hWnd,GraphPtr	pGraph,char FAR *pName, int nType);

int	NEAR  DoWriteFile(HWND hWnd,GraphPtr pGraph,char FAR *pName,int nType);

int	NEAR  GetInputFileName(HWND hWnd, char FAR *pFileName,INT16	nFileType);

int	NEAR  GetOutputFileName(HWND hWnd,char FAR *pFileName,INT16	nFileType);


/* PROGRAM SPECIFIC CALLS TO HOOK UP PG LIBRARY TO LOCAL DATA SOURCES */
BOOL PUBLIC InitDataCallBacks(HANDLE hInst, GraphPtr pGraph);
VOID PUBLIC FiniDataCallBacks(GraphPtr pGraph);
INT16 FAR PASCAL LOCAL_GetDataCallback(INT16	nDataPt,INT16 nSeries,
												  	REAL64 FAR * prVal,INT32 lClientData);

#define SetDefaultAreaColor(A,B)   (SetGraphAttrSL((A),NULL,A_AREACOLOR_RGB,(B)))
#define SetDefaultLineColor(A,B)   (SetGraphAttrSL((A),NULL,A_LINECOLOR_RGB,(B)))

/*--------------------------------------------------------------------------*\
|            	         Static Data.                                  		 |
\*--------------------------------------------------------------------------*/

HANDLE  hInst;
static MEMHDL hText;

/* PG - Global graph variables */
DrawEnvPtr	  gpDrawEnv;						/* ptr to StdDraw Environment */
GraphPtr		  gpGraph;							/* ptr to Graph Object */
int						gbGraphValid;					/* TRUE when graph exists */

SelListPtr		gpList;							/* ptr to Selection List */
DetNodeRef		gDetNode;						/* global DetNode			 */
DetNodeRefPtr	gpDetNode = &gDetNode;		/* ptr to global DetNode */
DetNodePtr		tempNode;						/* ptr to global DetNode */
SelItemPtr		gpSelItem;						/* ptr to Selection Item */
PolyHandle		ghPoly;							/* handle to global polygon struct*/
INT16					objType = OAN_RECTANGLE;
char	  			tmpbuf[256];
INT16					anodeID;							/* Annotation object ID code */
INT16	  			nStatus;
INT16					gTextAnodeID;
CHAR FileFilter[] = "TDL Files (*.TDL)\0*.TDL\0";

/* THE FOLLOWING STATIC VARIABLES ARE USED TO STORE THE CHART DATA */
#define MaxStringSz	64

char	StringData[11][MaxStringSz] = {"PG Sample Chart",
												 "by Three |D| Graphics",
												 "Dan", "Elmer","Jon",
												 "May", "June", "July",
												 "The Row Title", "The Column Title",
												 "A footnote"};

int   NumData [3][3] = {{300,400,500},
								{400,500,600},
								{500,600,700}};

char	 			gFileName[256];

char FAR *FileTypeExt[] = {
	"*.TDL",
	"*.BMP"
};

enum FileType {
	FILE_LOOK,
	FILE_BMP,
	MAX_FileType,
	FILE_LOOKANNO
};

WedgeInfoRec thisWedge;

/*--------------------------------------------------------------------------*\
|            	         Main Function: WinMain                        		 |
\*--------------------------------------------------------------------------*/

int PASCAL WinMain	(HANDLE hInstance,
							 HANDLE hPrevInstance,
							 LPSTR  lpszCmdLine,
							 int	  cmdShow)

	{
	HWND		hwnd;
	MSG		msg;
	WNDCLASS	wndclass;

	if (!hPrevInstance)
	  {
	  wndclass.lpszClassName = "SAMP9:MAIN";
	  wndclass.hInstance     = hInstance;
 	  wndclass.lpfnWndProc	 = RecWndProc;
	  wndclass.hCursor       = LoadCursor(NULL, IDC_ARROW);
	  wndclass.hIcon		    = LoadIcon (hInstance, "TDG_LOGO");
	  wndclass.lpszMenuName  = "SAMP9_MENU";
	  wndclass.hbrBackground = COLOR_WINDOW+1;
	  wndclass.style		    = CS_HREDRAW | CS_VREDRAW;
	  wndclass.cbClsExtra	 = 0;
	  wndclass.cbWndExtra	 = 0;

	  RegisterClass( &wndclass);
	  }

	hInst = hInstance;


	if ( nStatus = InitTDGLIB() ) 		/* PG - MAIN INITIALIZATION CALL */
	   {
		wsprintf(tmpbuf, "InitTDGLIB failed (%d)!", nStatus);
		MessageBox(NULL, tmpbuf, "WinMain",
					  MB_OK | MB_ICONHAND | MB_SYSTEMMODAL );
	   }

	hwnd = CreateWindow("SAMP9:MAIN", 			/* Class Name. */
							  "PG Sample #9",		/* Title			*/
							  WS_OVERLAPPEDWINDOW,	/* Style bits. */
							  CW_USEDEFAULT,			/* x - default */
							  0,							/* y - default	*/
							  CW_USEDEFAULT,			/* cx - default*/
							  0,							/* cy - default*/
							  NULL,						/* No parent	*/
							  NULL,						/* Class menu. */
							  hInstance,				/* Creator		*/
							  NULL						/* Params		*/
							 ) ;

	ShowWindow (hwnd, cmdShow);


   if ( !(gpGraph = AllocGraphPtr()) ) /* PG - ALLOCATE A GRAPH POINTER */
		return FALSE;
						
	/* THE NEXT CALL IS A LOCAL PROC THAT CALLS THE PG ROUTINES FOR
	   HOOKING UP A LOCAL DATA SOURCE TO THE PG LIBRARY */

	InitDataCallBacks(hInst, gpGraph);  

													/* PG - ALLOCATE DRAW ENVIRONMENT */
  if ( !(gpDrawEnv = AllocDrawEnvPtr(hwnd, DE_PORT_NORMAL)) )	
		return FALSE;
													/* PG- LOAD A BAR GRAPH TEMPLATE AS DEFAULT */
  nStatus = TDIReadGraph(gpGraph, REQUEST_TDL, "default.TDL");
	if (nStatus != E00_OK)
	   return FALSE;

													/* PG- CREATE AN EMPTY SELECTION LIST */
	gpList = Select_AllocList();


	/*
	** CREATE A HANDLE TO A DUMMY STRING 
	*/
	tmpbuf[0] = '\0';
	strcpy(tmpbuf,"Some Sample Text\0"); 
	Ptr2Handle((MEMPTR) tmpbuf, &hText, (MEMSIZE) strlen(tmpbuf)+1);

  UpdateWindow(hwnd);             /* Sends WM_PAINT message */

	while (GetMessage (&msg, 0, 0, 0))
		{
		TranslateMessage (&msg);	  /* Keyboard input. */
		DispatchMessage (&msg);
		}


	/* PERFORM PG CLEAN-UP */

	Select_FreeList (gpList);		 /* PG - FREE MEMORY USED BY THIS POINTER */

	FreeDrawEnvPtr(gpDrawEnv);		 /* PG - FREE MEMORY USED BY THIS POINTER */

	FiniDataCallBacks(gpGraph);	 /* LOCAL PROC - CLEAN UP THUNKS */
	
	FreeGraphPtr(gpGraph);			 /* PG - FREE MEMORY USED BY THIS POINTER */

  FiniTDGLIB(); 						 /* PG - CLEAN-UP */

	return 0;
	}


/*--------------------------------------------------------------------------*\
|            	         Window Procedure: RecWndProc.                 		 |
|   This is the main message processing loop for this application           |
\*--------------------------------------------------------------------------*/

long FAR PASCAL RecWndProc (HWND hWnd,		WORD msg,
									 WORD wParam,	LONG lParam)

	{								
	static BOOL fCreate  = TRUE;
	static BOOL fCapture = FALSE;
	static RECT	 r, aRect, rOld;
	static POINT p0, p1;
	static POINT ptDrag;
	static WORD	 wColor  = IDM_GREEN;
	static WORD	 wlColor  = IDM_LWHITE;
	static WORD	 wAnode  = IDM_RECT;
	static WORD	 wCreate = IDM_CREATE;
	static int	x,y;

 	/* PG - The following is a table of frequently used colors */

#define FULL 65535		/* PG - FULL INTENSITY OF A GIVEN COLOR */

	static RGB16 aRGB[5]  = {{   0,   0,   0},	/* BLACK */
									 {FULL,FULL,FULL},	/* WHITE */
									 {FULL,   0,   0},	/* RED   */
									 {   0,FULL,   0},	/* GREEN */
									 {   0,   0,FULL}};	/* BLUE  */

	HMENU	hmenu;
	char 	buffer[80];
	static int 	i = 0;


	switch (msg)
		{
		case WM_COMMAND:
			hmenu = GetMenu (hWnd);
			switch (wParam)
				{
 				case IDM_QUIT:			   /* TIME TO QUIT! */
					DestroyWindow (hWnd);
					break;

				case IDM_SAVE:
				  	if ( GetOutputFileName(hWnd, gFileName, FILE_LOOK) )
					  DoWriteFile(hWnd, gpGraph, gFileName, FILE_LOOK);
					break;

				case IDM_SAVEANNO:
				  	if ( GetOutputFileName(hWnd, gFileName, FILE_LOOK) )
					  DoWriteFile(hWnd, gpGraph, gFileName, FILE_LOOKANNO);
					break;

				case IDM_OPEN:
			  		if ( GetInputFileName(hWnd, gFileName, FILE_LOOK) )
					  gbGraphValid = (DoReadFile(hWnd, gpGraph, gFileName, FILE_LOOK) == E00_OK);
				   InvalidateRect (hWnd, &r, TRUE);
					break;

				case IDM_CREATE:
				case IDM_MODIFY:
					if (wParam == IDM_CREATE) 
					  fCreate = TRUE;
					else
					  fCreate = FALSE;
					UpdateMenu (hmenu,&wCreate,wParam);
					break;
				case IDM_TEXT_MODIFY:                       
					{
					char	szNewText[40]; 
					INT16	nLength = 40;
					
					/* If pString = NULL, function returns string length only */
					GetAnnotationText(gpGraph, gTextAnodeID, (char FAR *) NULL, &nLength); //Get Length of String;
					/* If we pass a string ptr, function will copy annotation string
						into szNewText (up to nLength chars will be copied) */
					GetAnnotationText(gpGraph, gTextAnodeID,szNewText,&nLength);    // Get String
					if (nLength < 35)
				 		strcat(szNewText," Also");
					SetAnnotationText(gpGraph, gpDrawEnv, gTextAnodeID, szNewText); // Set String
				   	InvalidateRect (hWnd, &r, TRUE);
					}
					break;

				case IDM_RRCT:	/* SET ANNOTATION OBJECT TYPE TO RECTANGLE */
					objType = OAN_ROUNDRECT;	
					UpdateMenu (hmenu,&wAnode,wParam);
					break;

				case IDM_RECT:
					objType = OAN_RECTANGLE;
					UpdateMenu (hmenu,&wAnode,wParam);
					break;

				case IDM_ELLIPSE:
					objType = OAN_ELLIPSE;
					UpdateMenu (hmenu,&wAnode,wParam);
					break;

				case IDM_LINE:
					objType = OAN_LINE;
					UpdateMenu (hmenu,&wAnode,wParam);
					break;

				case IDM_ARROW:
					objType = OAN_ARROW;
					UpdateMenu (hmenu,&wAnode,wParam);
					break;

				case IDM_TEXTBOX:
					{
					INT16	Left 			= ALIGN_LEFT;

					objType = OAN_TEXTBOX;

					/* PG - The following call will set the DEFAULT FONT alignment
					   to LEFT JUSTIFIED. So that when we draw the very first text
						 annotation, it will be left justified.  

						 NOTE: To modify an annotation attribute (AREA,LINE,FONT) before
						 drawing the object, simply pass NULL to the SelItem parameter
						 in SetGraphAttrSI.  This will modify the DEFAULT object (AREA,
						 LINE, or FONT) that the attributes use. 
					*/

	 			  SetGraphAttrSI(gpGraph,NULL,A_FONTALIGN, &Left); 
					UpdateMenu (hmenu,&wAnode,wParam);
					}
					break;

				case IDM_POLY:
					objType = OAN_POLYGON;
					UpdateMenu (hmenu,&wAnode,wParam);
					break;

				case IDM_WEDGE:
					objType = OAN_WEDGE;
					UpdateMenu (hmenu,&wAnode,wParam);
					break;

				case IDM_ARC:
					objType = OAN_ARC;
					UpdateMenu (hmenu,&wAnode,wParam);
					break;

			   case IDM_RED:							
			   case IDM_GREEN:		
			   case IDM_BLUE:
			   case IDM_BLACK:		
			   case IDM_WHITE:		

					UpdateMenu (hmenu,&wColor,wParam);

					/*
					** PG - SET CURRENT COLOR FOR ANNOTATION AREA FILLS 
					*/
					if (wParam == IDM_BLACK) 
					  SetDefaultAreaColor (gpGraph, &aRGB [0]); /*PG- MACRO FOR SetGraphAttr */
					if (wParam == IDM_WHITE) 
					  SetDefaultAreaColor (gpGraph, &aRGB [1]); /*PG- MACRO FOR SetGraphAttr */
					if (wParam == IDM_RED) 
					  SetDefaultAreaColor (gpGraph, &aRGB [2]); /*PG- MACRO FOR SetGraphAttr */
					if (wParam == IDM_GREEN)
					  SetDefaultAreaColor (gpGraph, &aRGB [3]); /*PG- MACRO FOR SetGraphAttr */
					if (wParam == IDM_BLUE) 
					  SetDefaultAreaColor (gpGraph, &aRGB [4]); /*PG- MACRO FOR SetGraphAttr */
					break;

			   case IDM_LRED:							
			   case IDM_LGREEN:		
			   case IDM_LBLUE:
			   case IDM_LBLACK:		
			   case IDM_LWHITE:		

					UpdateMenu (hmenu,&wlColor,wParam);

					/*
					** PG - SET CURRENT COLOR FOR ANNOTATION LINES & EDGES
					*/
					if (wParam == IDM_LBLACK) 
					  SetDefaultLineColor (gpGraph, &aRGB [0]); /*PG- MACRO FOR SetGraphAttr */
					if (wParam == IDM_LWHITE) 
					  SetDefaultLineColor (gpGraph, &aRGB [1]); /*PG- MACRO FOR SetGraphAttr */
					if (wParam == IDM_LRED) 
					  SetDefaultLineColor (gpGraph, &aRGB [2]); /*PG- MACRO FOR SetGraphAttr */
					if (wParam == IDM_LGREEN)
					  SetDefaultLineColor (gpGraph, &aRGB [3]); /*PG- MACRO FOR SetGraphAttr */
					if (wParam == IDM_LBLUE) 
					  SetDefaultLineColor (gpGraph, &aRGB [4]); /*PG- MACRO FOR SetGraphAttr */
					break;

				case IDM_DELETE:
					/*
					 * Annotation text boxes don't currently obey the Remove call.
					 * They need to be told explicitly to die 
					 * This will have to get fixed eventually (18dec91.ms).
					 */
//					if (gpSelItem->nObjectID == OTX_BOX_ANNOTATION)
//						ExecGraphFuncSL (gpDrawEnv, gpGraph, gpList, FTX_BOX_DELETE, NULL);
//					else
				   	Remove(gpGraph,gpList, FALSE);	/*PG - Kill item in list */

					InvalidateRect(hWnd, NULL, TRUE); 
					break;

				case IDM_DELETE_ALL:
				   Remove(gpGraph,NULL, TRUE);		/*PG - Kill all annot items*/
				   InvalidateRect(hWnd, NULL, TRUE); 
					break;

				case IDM_REDRAW:
				   InvalidateRect(hWnd, NULL, TRUE); 
					break;

				default:

					wsprintf (buffer, "Command = %d", wParam);
					MessageBox (hWnd, buffer, "WM_COMMAND", MB_OK);
				}
				break;

		case WM_LBUTTONDOWN:
			{
			int i;
      POINTS pts;
			POINT pt;
			
			if (fCreate)
			  {
			  SetCapture (hWnd);
			  fCapture = TRUE;

			  pts = MAKEPOINTS (lParam);
				pt.x = pts.x;
				pt.y = pts.y;
			  r.left = r.right = pt.x;
		    r.top = r.bottom = pt.y;
			  }
			else
			  {
				pts = MAKEPOINTS (lParam);		/* Get the Mouse Pt. */
				pt.x = pts.x;
				pt.y = pts.y;
		  	ptDrag = pt;
	  		dvPoint(gpDrawEnv,&pt);		/*PG - Convert to Virt. Coord's. */

		    if (FindDetNode (gpGraph, &pt, gpDetNode)) 
			    {
					Select_ClearList	(gpList);  /*PG - Clear our master list */
										  /*PG - Add this item to our list */
					gpSelItem = Select_AddItem(gpGraph,gpList,gpDetNode);
											  /*PG - Calculate Sizing Handles for item */
					Select_CalcHandlesItem(gpDrawEnv,gpSelItem);
	
					/*
					** PG - Easy way to see if selected object is movable 
					*/
					if (gpSelItem->nType != DETNODE)
					  {
					  r = gpSelItem->rcBounds;
					  vdRect(gpDrawEnv,&r);
					 	rOld = r;
				    fCapture = TRUE;
					  Rect2DrawNotRect (hWnd,
															r.left,
			   							  			r.top,
			  								 		  r.right,
			  								  		r.bottom);	
				    }
				  }
			  }
			}
			break;

		case WM_LBUTTONUP:
			{
		   int i;

			if (!fCapture) 
			  break;

			ReleaseCapture ();
			fCapture = FALSE;

	      /* Remove tracking rectangle */
	      Rect2DrawNotRect (hWnd,  r.left,
		   								 r.top,
		  									 r.right,
		  									 r.bottom);	
			if (fCreate) 
			  {
		  
		  	  /* Make sure order is correct: left < right */

		  	  if (r.left > r.right)
			    {
			    i = r.left;
			    r.left = r.right;
			    r.right = i;
			    }

		     /* Top should be less than bottom */
		     if (r.top > r.bottom)
			    {
			    i = r.top;
			    r.top = r.bottom;
			    r.bottom = i;
			    }

			  aRect = r;
			  dvRect(gpDrawEnv, &aRect);
			  p0.x = aRect.left;
			  p0.y = aRect.top;
			  p1.x = aRect.right;
			  p1.y = aRect.bottom;


			  if (objType == OAN_POLYGON)
			    {
			  /* DUMMY UP A POLYGON: A DIAMOND WITH WIDTH & HEIGHT
			  ** DETERMINED BY aRect				
			  */
				 ghPoly = AllocPolyHandle (2);	/*PG - MAKE POLY STRUCT */

				 x = (aRect.left+aRect.right)/2;
				 y = aRect.top;						/*PG - ADD FIRST POINT TO POLY */
			    nStatus = AddPolyPoint(&ghPoly, x, y, TRUE);	

			    x = aRect.left;
			    y = (aRect.top+aRect.bottom)/2; /*PG - ADD NEXT POINT TO POLY */
			    nStatus = AddPolyPoint(&ghPoly, x, y, FALSE);

				 x = (aRect.left+aRect.right)/2;
				 y = aRect.bottom;
			    nStatus = AddPolyPoint(&ghPoly, x, y, FALSE);

			    x = aRect.right;
			    y = (aRect.top+aRect.bottom)/2;
			    nStatus = AddPolyPoint(&ghPoly, x, y, FALSE);

				 x = (aRect.left+aRect.right)/2;
				 y = aRect.top;						/*PG - REPEAT FIRST POINT */
			    nStatus = AddPolyPoint(&ghPoly, x, y, FALSE);

				 ClosePolyHandle (&ghPoly); 		/*PG - CLOSE OFF POLYGON */

				 }

				if(objType == OAN_WEDGE)
				{
					thisWedge.bounds = aRect;		/* bounding rectangle for the wedge */
					/* wedge drawn with bounding rect from start ccw to end */
					thisWedge.ptStart.x = aRect.right;
					thisWedge.ptStart.y = aRect.top;
					thisWedge.ptEnd.x = aRect.left;
					thisWedge.ptEnd.y = aRect.top;
				}

				if(objType == OAN_ARC)
				{
					thisWedge.bounds = aRect;		/* bounding rectangle for the arc */
					/* wedge drawn with bounding rect from start ccw to end */
					thisWedge.ptStart.x = aRect.right;
					thisWedge.ptStart.y = aRect.bottom;
					thisWedge.ptEnd.x = aRect.left;
					thisWedge.ptEnd.y = aRect.bottom;
				}

			  /*
			  ** PG - MAIN CALL TO CREATE AN ANNOTATION 
			  */ 
			 if (objType == OAN_TEXTBOX)
		     	gTextAnodeID = CreateAnode(gpGraph, objType, &aRect, hText,
								   ghPoly, &thisWedge, p0, p1);			 
			 else
		     	CreateAnode(gpGraph, objType, &aRect, hText,
								   ghPoly, &thisWedge, p0, p1);



		     InvalidateRect (hWnd, &r, TRUE); 
			  }
			else
			  {
			  aRect = r;
			  dvRect(gpDrawEnv, &aRect);	/*PG - CONVERT TO VIRT. COORDS. */

  			  /*
			  ** PG - MAIN CALL TO MOVE SELECTED OBJECT
			  */
			  MoveSelectedItem(gpDrawEnv,gpGraph,gpSelItem,&aRect); /*NEW POSITION*/
		     InvalidateRect (hWnd, &rOld, TRUE); 			 /*UPDATE */
		     InvalidateRect (hWnd, &r, TRUE); 	  			 /*UPDATE */
			  }
			}

			break;

		case WM_MOUSEMOVE:
		   {

      POINTS pts;
			POINT pt;

			if (!fCapture)
			  break;

			pts = MAKEPOINTS (lParam);
			pt.x = pts.x;
			pt.y = pts.y;

	      /* Remove previous rectangle */
		   Rect2DrawNotRect (hWnd,r.left,
			                       r.top,
									     r.right,
									     r.bottom);
			if (fCreate) 
			  {

		     r.right = pt.x;
			   r.bottom = pt.y;


			  }
			else
			  {
			  /* Calculate relative movement. */
			  ptDrag.x = ptDrag.x - pt.x;    
			  ptDrag.y = ptDrag.y - pt.y;    

			  r.left   -= ptDrag.x;
			  r.top    -= ptDrag.y;
			  r.right  -= ptDrag.x;
			  r.bottom -= ptDrag.y;

			  /* Remember drag point for next time. */

			  ptDrag = pt;


			  }
		   /* Draw a new rectangle */
			Rect2DrawNotRect (hWnd,r.left,
			                       r.top,
									     r.right,
									     r.bottom);

			}
			break;

		case WM_DESTROY:
			PostQuitMessage (0);
			break;

		case WM_PAINT:
			{
			PAINTSTRUCT ps;
			HDC hdc;
			int i;

			hdc = BeginPaint (hWnd, &ps);

	   	/*PG - SET GRAPH REGION TO ENTIRE CLIENT AREA */
			    GetClientRect(hWnd, &r);
			    DESetDestRect(gpDrawEnv, &r);

  	 	/*PG - SET DRAWING MODE TO NORMAL, ON-SCREEN */
			    DESetPortInfo(gpDrawEnv, hdc, DE_PORT_NORMAL);

	  	/*PG - DRAW THE GRAPH */
	        DrawBegin(gpDrawEnv);
		        DrawTheGraph(gpDrawEnv, gpGraph, TRUE, TRUE, TRUE);
			    DrawEnd(gpDrawEnv);

		  /*PG - Match up objects in selection list with new detnodes */
					Select_RebuildList (gpGraph,gpList);

	   	/*PG - Calculate handles for objects in selection list based on new detnodes*/
					Select_CalcHandlesList (gpDrawEnv,gpList);

		  EndPaint (hWnd, &ps);

		  }
			break;

		default:
			return(DefWindowProc(hWnd,msg,wParam,lParam));
			break;
		}
	return 0L;
	}

/*--------------------------------------------------------------------------*\
|            	  Rect2DrawNotRect: Draw a NOT Rectangle               		 |
\*--------------------------------------------------------------------------*/

void NEAR PASCAL Rect2DrawNotRect (HWND hWnd, int x1, int y1,
                                   int x2, int y2)
	{
	HDC hdc;

	hdc = GetDC (hWnd);
	SelectObject (hdc, GetStockObject (NULL_BRUSH));
	SetROP2 (hdc, R2_NOT);
	Rectangle (hdc, x1, y1, x2, y2);
	ReleaseDC (hWnd,hdc);
	}



/*-------------------DATA-CALLBACK-ROUTINES---------------------------------*/

/*--------------------------------------------------------------------------*\
|            	  LOCAL_GetStringCallback.						              		 |
| Gets asked by PG for a particular piece of legend text. Returns the 	 |
| correct string by looking into the StringData array.                      |   
| The gs constants (gsTITLE,gsSUBTITLE) are enumerated in PG.H				 |
\*--------------------------------------------------------------------------*/

INT16 FAR PASCAL  
LOCAL_GetStringCallback(
	INT16						nWhich,							/* Which legend?. */
	char FAR *			   lpBuffer,						/* Put answer here */
	INT16						nDimBuffer,						/* Max.size of answer */
	INT32						lClientData						/* client data */
)
{
switch (nWhich)
  {
  case gsTITLE:
    lstrcpy (lpBuffer,StringData[0]);	/* RETURN THE TITLE OF THE GRAPH */
	 break;
  case gsSUBTITLE:							/* RETURN THE SUBTITLE OF THE GRAPH */
    lstrcpy (lpBuffer,StringData[1]);
	 break;
  case gsFOOTNOTE:							/* RETURN THE GRAPH FOOTONOTE */
    lstrcpy (lpBuffer,StringData[10]);
	 break;
  case gsSERIESTITLE:						/* RETURN THE SERIES TITLE */
    lstrcpy (lpBuffer,StringData[9]);
	 break;
  case gsGROUPSTITLE:						/* RETURN THE GROUP TITLE */
    lstrcpy (lpBuffer,StringData[8]);
	 break;
  default:										/* PUT "XXXX" IN ANY OTHER REQUEST */
    lstrcpy (lpBuffer,"XXXX");
  }
return TRUE;
}


/*--------------------------------------------------------------------------*\
|            	  LOCAL_GetLabelCallback.						              		 |
| Gets asked by PG either a row or column header string. Returns the 	    |
| correct string by looking into the StringData array.                      |   
\*--------------------------------------------------------------------------*/
INT16 FAR PASCAL 
LOCAL_GetLabelCallback(
	INT16						nCol,				/* Set to -1 when traversing Row headers */
	INT16						nRow,				/* Set to -1 when traversing Col headers */ 
	char FAR *		   	lpBuffer,		/* Put answer here. */
	INT16						nDimBuffer,		/* Max.size of answer. */
	INT32						lClientData		/* client data value */
)
{
int i;

if (nCol == -1) 	/* THIS MEANS WE'RE TRAVERSING ROW HEADERS */
  i = 2 + nRow;
else
  if (nRow == -1)	/* THIS MEANS WE'RE TRAVERSING COL HEADERS */
    i = 5 + nCol; /* OTHERWISE WE'RE TRAVERSING COLUMN HEADERS */
  else
    i = 10;			/* ANY OTHER REQUEST GETS THE FOOTNOTE STRING */

lstrcpy (lpBuffer,StringData[i]);	/* RETURN THE CORRECT HEADER */

return TRUE;
}


/*--------------------------------------------------------------------------*\
|            	  LOCAL_GetDataCallback.						              		    |
| Gets asked by PG for a piece of numeric data corresponding to a			 |
| particular row & column offset.  Returns the 	                            |
| correct value by looking into the NumData array.                          |   
\*--------------------------------------------------------------------------*/

INT16 FAR PASCAL 
LOCAL_GetDataCallback(
	INT16						nDataPt,							/* Row number of requested value. */
	INT16						nSeries,							/* Column number of requested value. */
	REAL64 FAR *		   prVal,							/* Put answer here   */
	INT32						lClientData						/* client data value */
)
{
*prVal = (REAL64) NumData [nDataPt][nSeries];      /* RETURN THE VALUE */
return TRUE;
}


/*--------------------------------------------------------------------------*\
|            	  LOCAL_GetDataCallback.						          		    |
| Utility function that formats a numeric value of type REAL64 into a		 |
| string. Not used by any of the graphs in this program.							 |
\*--------------------------------------------------------------------------*/

INT16 FAR PASCAL 
LOCAL_RealToStrCallback(
	REAL64					rVal,								/* Value to be formatted. */
	char FAR *			lpszResult,							/* Caller provided buffer for string assigment. */
	INT16						nDimBuffer,						/* Buffer length. */
	INT16						nFmtCode,						/* Format conversion identifier. */
	INT32						lClientData						/* client data value */
)
{
wsprintf(lpszResult,"%d",(int) rVal);	 		/* JUST DO SIMPLE INT FORMAT */
return TRUE;
}

/*--------------------------------------------------------------------------*\
|            	  LOCAL_SetPageCallback.						           		    |
| Not actually used by current PG, but needed for expansion.  For now,    |
| just return FALSE 																			 |
\*--------------------------------------------------------------------------*/

INT16 FAR PASCAL 
LOCAL_SetPageCallback(
	INT16						nPage,								/* New current page. */
	INT32						lClientData						/* client data value */
)
{
return FALSE;
}

/************************************************************************
* 	InitDataCallBacks
*
*	DESCRIPTION:
*		This function connects the above CallBack functions with a 
*		particular graph.
*		
*	RETURNS:
*		FALSE if any of the callback functions are set to NULL.
*
************************************************************************/

BOOL PUBLIC InitDataCallBacks(HANDLE hInst, GraphPtr pGraph)
{
	GraphDataInfo		graphData;
	
	/*
	** PG - Look up Data information connected to this graph
   */
	GetGraphDataInfo(pGraph, &graphData);
	
	/*
	** If the data information doesn't exist, create thunks as necessary
	*/
	if (!graphData.lpfnGetData)
	   { 
		graphData.lpfnGetString    = (LPFNGetStringCallBack)    MakeProcInstance((FARPROC) LOCAL_GetStringCallback, hInst);
		graphData.lpfnGetLabel     = (LPFNGetLabelCallBack)     MakeProcInstance((FARPROC) LOCAL_GetLabelCallback,  hInst);
		graphData.lpfnGetData      = (LPFNGetDataCallBack)      MakeProcInstance((FARPROC) LOCAL_GetDataCallback,   hInst);
		graphData.lpfnSetPage      = (LPFNSetPageCallBack)      MakeProcInstance((FARPROC) LOCAL_SetPageCallback,   hInst);
		graphData.lpfnRealToString = (LPFNRealToStringCallBack) MakeProcInstance((FARPROC) LOCAL_RealToStrCallback, hInst);
		graphData.lpfnGetRawLimits = NULL; /* If not using MUST set to NULL */
		graphData.lpfnPicture      = NULL; /* If not using MUST set to NULL */
	   }

	/* 
  ** Good practice to always explicitly set Data Interface Mode
  */
	graphData.nDataIFMode = DATAIF_CALLBACK; 	/* Use Callback Data Interface method */

   
	/*
	** always set datalook-related info
	*/
	graphData.nPages      = 1;	 	/* THIS SHOULD ALWAYS BE SET TO 1 */
	graphData.nRows       = 3;		/* SET TO NUMBER OF ROWS TO GRAPH */
	graphData.nCols       = 3;		/* SET TO NUMBER OF COLUMNS TO GRAPH */
		/* PUT HANDLE TO THIS GRAPH INTO A FIELD THAT IS PASSED TO EVERY
		   DATA CALLBACK FUNCTION.  THIS LETS THE FUNCTION KNOW WHICH GRAPH
			IS DOING THE CALLING. */
	graphData.lClientData = (INT32) pGraph;	

	/* 
	** PG - Register the callback functions with the PG library 
	*/
	return SetGraphDataInfo(pGraph, &graphData);
}


/************************************************************************
* 	FiniDataCallBacks
*
*	DESCRIPTION:
*		This function dis-connects the above CallBack functions with a 
*		particular graph.
*		
*	RETURNS:
*		nothing.
*
************************************************************************/
VOID PUBLIC FiniDataCallBacks(GraphPtr pGraph)
{
	GraphDataInfo		graphData;
	
	GetGraphDataInfo(pGraph, &graphData);

	if (graphData.lpfnGetString) 
	  {
		FreeProcInstance( graphData.lpfnGetString    );
		FreeProcInstance( graphData.lpfnGetLabel     );
		FreeProcInstance( graphData.lpfnGetData      );
		FreeProcInstance( graphData.lpfnSetPage      );
		FreeProcInstance( graphData.lpfnRealToString );
	  }
	
}


/*--------------------------------------------------------------------------*\
|            	  UpdateMenu: Update Checkmarks on main menu               	 |
\*--------------------------------------------------------------------------*/

void NEAR PASCAL UpdateMenu (HMENU hmenu, WORD *mVar, WORD wParam)
	{
	CheckMenuItem(hmenu,*mVar,MF_BYCOMMAND | MF_UNCHECKED);
	*mVar = wParam;
	CheckMenuItem(hmenu,*mVar,MF_BYCOMMAND | MF_CHECKED);
	}

/************************************************************************
*	DoReadFile
*
*	DESCRIPTION:
*		Does a bunch of stuff related to reading a new 3Dx file.
*
*	RETURNS:
*
************************************************************************/
int NEAR
DoReadFile(
	HWND						hWnd,
	GraphPtr				pGraph,
	char FAR *			pName,
	INT16						nType
)
{
	int							nResult;
	char						tmpbuf[256];

	SetCursor (LoadCursor (NULL, IDC_WAIT));
	
	/*
	 *  kill any existing DetNodes
	 */
	FreeAllDetNodes(pGraph);

	/*
	 *  kill the annotation stuff too
	 */
	FreeAllAnodes(pGraph);
	
	switch (nType) {

#if 0
										
	case FILE_3D3:			/*	This is usable only in the VDM version of PG */
	  nResult = TDIReadGraph(pGraph, REQUEST_3D3, pName);
		break;
#endif

	case FILE_LOOK:
	  nResult = TDIReadGraph(pGraph, REQUEST_TDL, pName);
		break;
	
	default:
		nResult = -1;
		break;
	}

	InvalidateRect(hWnd, NULL, TRUE);

	/* set the window title too */
	wsprintf(tmpbuf, "PG SAMPLE #6 - %s", pName);
	SetWindowText(hWnd, tmpbuf);

	SetCursor (LoadCursor (NULL, IDC_ARROW));
  
	/* return the error code */
	return nResult;
}



/************************************************************************
*	DoWriteFile
*
*	DESCRIPTION:
*		Does a bunch of stuff related to reading a new 3Dx file.
*
*	RETURNS:
*
************************************************************************/
int NEAR
DoWriteFile(
	HWND						hWnd,
	GraphPtr				pGraph,
	char FAR *			pName,
	int							nType
)
{
	int							nResult;
	
	SetCursor (LoadCursor (NULL, IDC_WAIT));
	switch (nType) {

#if 0
	case FILE_3D3:			/*  This is usable only in the VDM version of PG  */
	  nResult = TDIWriteGraph(pGraph, REQUEST_3D3, pName);
		break;
#endif

	case FILE_LOOK:	
	  nResult = TDIWriteGraph(pGraph, REQUEST_TDL, pName);
		break;

	case FILE_LOOKANNO:	
	  nResult = TDIWriteGraph(pGraph, REQUEST_TDLANNOTATION, pName);
		break;

	default:
		nResult = -1;
		break;
	}
	SetCursor (LoadCursor (NULL, IDC_ARROW));
	
	return nResult;
}


/************************************************************************
*	GetInputFileName
*
*	DESCRIPTION:
*		Requests the user to choose an existing file.
*
*	RETURNS:
*		TRUE if user chose something
*
************************************************************************/
int NEAR
GetInputFileName(
	HWND						hWnd,
	char FAR *			pFileName,
	INT16						nFileType
)
{
  int              nResult;
  char            fName[256];
  OPENFILENAME	   OpenFileName;


  /*** clone filename to prevent damage ***/
  lstrcpy(fName, pFileName);
  

  /*** Call DlgOpen() to get the filename ***/

#if 0
  nResult = DlgOpenFile( hWnd,
                        "Select an Input File",
                        (LONG) OF_EXIST | OF_MUSTEXIST | OF_NOOPTIONS,
                        FileTypeExt[nFileType],
                        fName,
                        NULL);
#else
   OpenFileName.lStructSize       = sizeof(OPENFILENAME);
   OpenFileName.hwndOwner         = hWnd;
   OpenFileName.hInstance         = (HANDLE) hInst;
   OpenFileName.lpstrFilter       = FileFilter;
   OpenFileName.lpstrCustomFilter = (LPSTR) NULL;
   OpenFileName.nMaxCustFilter    = 0L;
   OpenFileName.nFilterIndex      = 1L;
   OpenFileName.lpstrFile         = fName;
   OpenFileName.nMaxFile          = sizeof(fName);
   OpenFileName.lpstrFileTitle    = (LPSTR) NULL;
   OpenFileName.nMaxFileTitle     = 0L;
   OpenFileName.lpstrInitialDir   = NULL;
   OpenFileName.lpstrTitle        = "Select an Input File";
   OpenFileName.nFileOffset       = 0;
   OpenFileName.nFileExtension    = 0;
   OpenFileName.lpstrDefExt       = FileTypeExt[nFileType];
   OpenFileName.lCustData         = 0;
   OpenFileName.Flags = OFN_SHOWHELP | OFN_PATHMUSTEXIST |
       OFN_FILEMUSTEXIST;

	nResult = GetOpenFileName(&OpenFileName);
#endif

  if (nResult)
    lstrcpy(pFileName, fName);

  return (nResult > 0);
}


/************************************************************************
* 	GetOutputFileName
*
*	DESCRIPTION:
*		Requests the user to choose an existing file.
*
*	RETURNS:
*		TRUE if user chose something
*
************************************************************************/
int NEAR
GetOutputFileName(
	HWND						hWnd,
	char FAR *			pFileName,
	INT16						nFileType
)
{
  int              nResult;
  char            fName[256];
	OPENFILENAME				OpenFileName;

  /*** clone filename to prevent damage ***/
  lstrcpy(fName, pFileName);
  
  /* Call DlgOpen() to set the filename */
  
#if 0  
  nResult = DlgOpenFile( hWnd,
                        "Enter Output Filename",
                        (LONG) OF_SAVE | OF_SHOWSPEC | OF_NOOPTIONS,
                        FileTypeExt[nFileType],
                        fName,
                        NULL);

#else
   OpenFileName.lStructSize       = sizeof(OPENFILENAME);
   OpenFileName.hwndOwner         = hWnd;
   OpenFileName.hInstance         = (HANDLE) hInst;
   OpenFileName.lpstrFilter       = (LPSTR) NULL;
   OpenFileName.lpstrCustomFilter = (LPSTR) NULL;
   OpenFileName.nMaxCustFilter    = 0L;
   OpenFileName.nFilterIndex      = 0L;
   OpenFileName.lpstrFile         = fName;
   OpenFileName.nMaxFile          = sizeof(fName);
   OpenFileName.lpstrFileTitle    = (LPSTR) NULL;
   OpenFileName.nMaxFileTitle     = 0L;
   OpenFileName.lpstrInitialDir   = NULL;
   OpenFileName.lpstrTitle        = "Enter Output Filename";
   OpenFileName.nFileOffset       = 0;
   OpenFileName.nFileExtension    = 0;
   OpenFileName.lpstrDefExt       = FileTypeExt[nFileType];
   OpenFileName.lCustData         = 0;
   OpenFileName.Flags = 0L;

	nResult = GetSaveFileName(&OpenFileName);
#endif
    
  if (nResult)
    lstrcpy(pFileName, fName);

	return  1; /* (nResult >= 0);*/
}

